@@ -25,6 +25,7 @@
 #include <time.h>
 #include <string.h>
 #include <limits.h>
+#include <fcntl.h>
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <unistd.h>
@@ -37,82 +38,76 @@
 
 /* libalpm */
 #include "add.h"
+#include "alpm.h"
 #include "alpm_list.h"
+#include "handle.h"
 #include "trans.h"
 #include "util.h"
 #include "log.h"
 #include "backup.h"
 #include "package.h"
 #include "db.h"
-#include "conflict.h"
-#include "deps.h"
 #include "remove.h"
 #include "handle.h"
 
-/** Add a package to the transaction.
- * @param pkg the package to add
- * @return 0 on success, -1 on error (pm_errno is set accordingly)
- */
-int SYMEXPORT alpm_add_pkg(pmpkg_t *pkg)
+/** Add a package to the transaction. */
+int SYMEXPORT alpm_add_pkg(alpm_handle_t *handle, alpm_pkg_t *pkg)
 {
 	const char *pkgname, *pkgver;
-	pmtrans_t *trans;
-	pmdb_t *db_local;
-	pmpkg_t *local;
-
-	ALPM_LOG_FUNC;
+	alpm_trans_t *trans;
+	alpm_pkg_t *local;
 
 	/* Sanity checks */
-	ASSERT(pkg != NULL, RET_ERR(PM_ERR_WRONG_ARGS, -1));
-	ASSERT(handle != NULL, RET_ERR(PM_ERR_HANDLE_NULL, -1));
+	CHECK_HANDLE(handle, return -1);
+	ASSERT(pkg != NULL, RET_ERR(handle, ALPM_ERR_WRONG_ARGS, -1));
+	ASSERT(handle == pkg->handle, RET_ERR(handle, ALPM_ERR_WRONG_ARGS, -1));
 	trans = handle->trans;
-	ASSERT(trans != NULL, RET_ERR(PM_ERR_TRANS_NULL, -1));
-	ASSERT(trans->state == STATE_INITIALIZED, RET_ERR(PM_ERR_TRANS_NOT_INITIALIZED, -1));
-	ASSERT(trans != NULL, RET_ERR(PM_ERR_TRANS_NULL, -1));
-	db_local = handle->db_local;
+	ASSERT(trans != NULL, RET_ERR(handle, ALPM_ERR_TRANS_NULL, -1));
+	ASSERT(trans->state == STATE_INITIALIZED,
+			RET_ERR(handle, ALPM_ERR_TRANS_NOT_INITIALIZED, -1));
 
-	pkgname = alpm_pkg_get_name(pkg);
-	pkgver = alpm_pkg_get_version(pkg);
+	pkgname = pkg->name;
+	pkgver = pkg->version;
 
-	_alpm_log(PM_LOG_DEBUG, "adding package '%s'\n", pkgname);
+	_alpm_log(handle, ALPM_LOG_DEBUG, "adding package '%s'\n", pkgname);
 
 	if(_alpm_pkg_find(trans->add, pkgname)) {
-		RET_ERR(PM_ERR_TRANS_DUP_TARGET, -1);
+		RET_ERR(handle, ALPM_ERR_TRANS_DUP_TARGET, -1);
 	}
 
-	local = _alpm_db_get_pkgfromcache(db_local, pkgname);
+	local = _alpm_db_get_pkgfromcache(handle->db_local, pkgname);
 	if(local) {
-		const char *localpkgname = alpm_pkg_get_name(local);
-		const char *localpkgver = alpm_pkg_get_version(local);
+		const char *localpkgname = local->name;
+		const char *localpkgver = local->version;
 		int cmp = _alpm_pkg_compare_versions(pkg, local);
 
 		if(cmp == 0) {
-			if(trans->flags & PM_TRANS_FLAG_NEEDED) {
+			if(trans->flags & ALPM_TRANS_FLAG_NEEDED) {
 				/* with the NEEDED flag, packages up to date are not reinstalled */
-				_alpm_log(PM_LOG_WARNING, _("%s-%s is up to date -- skipping\n"),
+				_alpm_log(handle, ALPM_LOG_WARNING, _("%s-%s is up to date -- skipping\n"),
 						localpkgname, localpkgver);
-				return(0);
-			} else if(!(trans->flags & PM_TRANS_FLAG_DOWNLOADONLY)) {
-				_alpm_log(PM_LOG_WARNING, _("%s-%s is up to date -- reinstalling\n"),
+				return 0;
+			} else if(!(trans->flags & ALPM_TRANS_FLAG_DOWNLOADONLY)) {
+				_alpm_log(handle, ALPM_LOG_WARNING, _("%s-%s is up to date -- reinstalling\n"),
 						localpkgname, localpkgver);
 			}
 		} else if(cmp < 0) {
 			/* local version is newer */
-			_alpm_log(PM_LOG_WARNING, _("downgrading package %s (%s => %s)\n"),
+			_alpm_log(handle, ALPM_LOG_WARNING, _("downgrading package %s (%s => %s)\n"),
 					localpkgname, localpkgver, pkgver);
 		}
 	}
 
 	/* add the package to the transaction */
-	pkg->reason = PM_PKG_REASON_EXPLICIT;
-	_alpm_log(PM_LOG_DEBUG, "adding package %s-%s to the transaction targets\n",
+	pkg->reason = ALPM_PKG_REASON_EXPLICIT;
+	_alpm_log(handle, ALPM_LOG_DEBUG, "adding package %s-%s to the transaction add list\n",
 						pkgname, pkgver);
 	trans->add = alpm_list_add(trans->add, pkg);
 
-	return(0);
+	return 0;
 }
 
-static int perform_extraction(struct archive *archive,
+static int perform_extraction(alpm_handle_t *handle, struct archive *archive,
 		struct archive_entry *entry, const char *filename, const char *origname)
 {
 	int ret;
@@ -125,27 +120,26 @@ static int perform_extraction(struct archive *archive,
 	ret = archive_read_extract(archive, entry, archive_flags);
 	if(ret == ARCHIVE_WARN && archive_errno(archive) != ENOSPC) {
 		/* operation succeeded but a "non-critical" error was encountered */
-		_alpm_log(PM_LOG_WARNING, _("warning given when extracting %s (%s)\n"),
+		_alpm_log(handle, ALPM_LOG_WARNING, _("warning given when extracting %s (%s)\n"),
 				origname, archive_error_string(archive));
 	} else if(ret != ARCHIVE_OK) {
-		_alpm_log(PM_LOG_ERROR, _("could not extract %s (%s)\n"),
+		_alpm_log(handle, ALPM_LOG_ERROR, _("could not extract %s (%s)\n"),
 				origname, archive_error_string(archive));
-		alpm_logaction("error: could not extract %s (%s)\n",
+		alpm_logaction(handle, "error: could not extract %s (%s)\n",
 				origname, archive_error_string(archive));
-		return(1);
+		return 1;
 	}
-	return(0);
+	return 0;
 }
 
-static int extract_single_file(struct archive *archive,
-		struct archive_entry *entry, pmpkg_t *newpkg, pmpkg_t *oldpkg,
-		pmtrans_t *trans, pmdb_t *db)
+static int extract_single_file(alpm_handle_t *handle, struct archive *archive,
+		struct archive_entry *entry, alpm_pkg_t *newpkg, alpm_pkg_t *oldpkg)
 {
 	const char *entryname;
 	mode_t entrymode;
 	char filename[PATH_MAX]; /* the actual file we're extracting */
 	int needbackup = 0, notouch = 0;
-	char *hash_orig = NULL;
+	const char *hash_orig = NULL;
 	char *entryname_orig = NULL;
 	int errors = 0;
 
@@ -157,19 +151,19 @@ static int extract_single_file(struct archive *archive,
 	if(strcmp(entryname, ".INSTALL") == 0) {
 		/* the install script goes inside the db */
 		snprintf(filename, PATH_MAX, "%s%s-%s/install",
-				_alpm_db_path(db), newpkg->name, newpkg->version);
+				_alpm_db_path(handle->db_local), newpkg->name, newpkg->version);
 		archive_entry_set_perm(entry, 0644);
 	} else if(strcmp(entryname, ".CHANGELOG") == 0) {
 		/* the changelog goes inside the db */
 		snprintf(filename, PATH_MAX, "%s%s-%s/changelog",
-				_alpm_db_path(db), newpkg->name, newpkg->version);
+				_alpm_db_path(handle->db_local), newpkg->name, newpkg->version);
 		archive_entry_set_perm(entry, 0644);
 	} else if(*entryname == '.') {
 		/* for now, ignore all files starting with '.' that haven't
 		 * already been handled (for future possibilities) */
-		_alpm_log(PM_LOG_DEBUG, "skipping extraction of '%s'\n", entryname);
+		_alpm_log(handle, ALPM_LOG_DEBUG, "skipping extraction of '%s'\n", entryname);
 		archive_read_data_skip(archive);
-		return(0);
+		return 0;
 	} else {
 		/* build the new entryname relative to handle->root */
 		snprintf(filename, PATH_MAX, "%s%s", handle->root, entryname);
@@ -177,12 +171,12 @@ static int extract_single_file(struct archive *archive,
 
 	/* if a file is in NoExtract then we never extract it */
 	if(alpm_list_find_str(handle->noextract, entryname)) {
-		_alpm_log(PM_LOG_DEBUG, "%s is in NoExtract, skipping extraction\n",
+		_alpm_log(handle, ALPM_LOG_DEBUG, "%s is in NoExtract, skipping extraction\n",
 				entryname);
-		alpm_logaction("note: %s is in NoExtract, skipping extraction\n",
+		alpm_logaction(handle, "note: %s is in NoExtract, skipping extraction\n",
 				entryname);
 		archive_read_data_skip(archive);
-		return(0);
+		return 0;
 	}
 
 	/* Check for file existence. This is one of the more crucial parts
@@ -216,42 +210,42 @@ static int extract_single_file(struct archive *archive,
 				if(lsbuf.st_mode != entrymode) {
 					/* if filesystem perms are different than pkg perms, warn user */
 					mode_t mask = 07777;
-					_alpm_log(PM_LOG_WARNING, _("directory permissions differ on %s\n"
+					_alpm_log(handle, ALPM_LOG_WARNING, _("directory permissions differ on %s\n"
 								"filesystem: %o  package: %o\n"), entryname, lsbuf.st_mode & mask,
 							entrymode & mask);
-					alpm_logaction("warning: directory permissions differ on %s\n"
+					alpm_logaction(handle, "warning: directory permissions differ on %s\n"
 							"filesystem: %o  package: %o\n", entryname, lsbuf.st_mode & mask,
 							entrymode & mask);
 				}
-				_alpm_log(PM_LOG_DEBUG, "extract: skipping dir extraction of %s\n",
+				_alpm_log(handle, ALPM_LOG_DEBUG, "extract: skipping dir extraction of %s\n",
 						entryname);
 				archive_read_data_skip(archive);
-				return(0);
+				return 0;
 			} else {
 				/* case 10/11: trying to overwrite dir with file/symlink, don't allow it */
-				_alpm_log(PM_LOG_ERROR, _("extract: not overwriting dir with file %s\n"),
+				_alpm_log(handle, ALPM_LOG_ERROR, _("extract: not overwriting dir with file %s\n"),
 						entryname);
 				archive_read_data_skip(archive);
-				return(1);
+				return 1;
 			}
 		} else if(S_ISLNK(lsbuf.st_mode) && S_ISDIR(entrymode)) {
 			/* case 9: existing symlink, dir in package */
 			if(S_ISDIR(sbuf.st_mode)) {
 				/* the symlink on FS is to a directory, so we'll use it */
-				_alpm_log(PM_LOG_DEBUG, "extract: skipping symlink overwrite of %s\n",
+				_alpm_log(handle, ALPM_LOG_DEBUG, "extract: skipping symlink overwrite of %s\n",
 						entryname);
 				archive_read_data_skip(archive);
-				return(0);
+				return 0;
 			} else {
 				/* this is BAD. symlink was not to a directory */
-				_alpm_log(PM_LOG_ERROR, _("extract: symlink %s does not point to dir\n"),
+				_alpm_log(handle, ALPM_LOG_ERROR, _("extract: symlink %s does not point to dir\n"),
 						entryname);
 				archive_read_data_skip(archive);
-				return(1);
+				return 1;
 			}
 		} else if(S_ISREG(lsbuf.st_mode) && S_ISDIR(entrymode)) {
 			/* case 6: trying to overwrite file with dir */
-			_alpm_log(PM_LOG_DEBUG, "extract: overwriting file with dir %s\n",
+			_alpm_log(handle, ALPM_LOG_DEBUG, "extract: overwriting file with dir %s\n",
 					entryname);
 		} else if(S_ISREG(entrymode)) {
 			/* case 4,7: */
@@ -259,24 +253,24 @@ static int extract_single_file(struct archive *archive,
 			if(alpm_list_find_str(handle->noupgrade, entryname)) {
 				notouch = 1;
 			} else {
+				alpm_backup_t *backup;
 				/* go to the backup array and see if our conflict is there */
 				/* check newpkg first, so that adding backup files is retroactive */
-				if(alpm_list_find_str(alpm_pkg_get_backup(newpkg), entryname) != NULL) {
+				backup = _alpm_needbackup(entryname, newpkg);
+				if(backup) {
+					/* if we force hash_orig to be non-NULL retroactive backup works */
+					hash_orig = "";
 					needbackup = 1;
 				}
 
 				/* check oldpkg for a backup entry, store the hash if available */
 				if(oldpkg) {
-					hash_orig = _alpm_needbackup(entryname, alpm_pkg_get_backup(oldpkg));
-					if(hash_orig) {
+					backup = _alpm_needbackup(entryname, oldpkg);
+					if(backup) {
+						hash_orig = backup->hash;
 						needbackup = 1;
 					}
 				}
-
-				/* if we force hash_orig to be non-NULL retroactive backup works */
-				if(needbackup && !hash_orig) {
-					STRDUP(hash_orig, "", RET_ERR(PM_ERR_MEMORY, -1));
-				}
 			}
 		}
 		/* else if(S_ISLNK(entrymode)) */
@@ -285,7 +279,7 @@ static int extract_single_file(struct archive *archive,
 
 	/* we need access to the original entryname later after calls to
 	 * archive_entry_set_pathname(), so we need to dupe it and free() later */
-	STRDUP(entryname_orig, entryname, RET_ERR(PM_ERR_MEMORY, -1));
+	STRDUP(entryname_orig, entryname, RET_ERR(handle, ALPM_ERR_MEMORY, -1));
 
 	if(needbackup) {
 		char checkfile[PATH_MAX];
@@ -294,44 +288,36 @@ static int extract_single_file(struct archive *archive,
 
 		snprintf(checkfile, PATH_MAX, "%s.paccheck", filename);
 
-		ret = perform_extraction(archive, entry, checkfile, entryname_orig);
+		ret = perform_extraction(handle, archive, entry, checkfile, entryname_orig);
 		if(ret == 1) {
 			/* error */
-			FREE(hash_orig);
 			FREE(entryname_orig);
-			return(1);
+			return 1;
 		}
 
 		hash_local = alpm_compute_md5sum(filename);
 		hash_pkg = alpm_compute_md5sum(checkfile);
 
-		/* append the new md5 hash to it's respective entry
-		 * in newpkg's backup (it will be the new orginal) */
-		alpm_list_t *backups;
-		for(backups = alpm_pkg_get_backup(newpkg); backups;
-				backups = alpm_list_next(backups)) {
-			char *oldbackup = alpm_list_getdata(backups);
-			if(!oldbackup || strcmp(oldbackup, entryname_orig) != 0) {
+		/* update the md5 hash in newpkg's backup (it will be the new orginal) */
+		alpm_list_t *i;
+		for(i = alpm_pkg_get_backup(newpkg); i; i = i->next) {
+			alpm_backup_t *backup = i->data;
+			char *newhash;
+			if(!backup->name || strcmp(backup->name, entryname_orig) != 0) {
 				continue;
 			}
-			char *backup = NULL;
-			/* length is tab char, null byte and MD5 (32 char) */
-			size_t backup_len = strlen(oldbackup) + 34;
-			MALLOC(backup, backup_len, RET_ERR(PM_ERR_MEMORY, -1));
-
-			sprintf(backup, "%s\t%s", oldbackup, hash_pkg);
-			backup[backup_len-1] = '\0';
-			FREE(oldbackup);
-			backups->data = backup;
+			STRDUP(newhash, hash_pkg, RET_ERR(handle, ALPM_ERR_MEMORY, -1));
+			FREE(backup->hash);
+			backup->hash = newhash;
 		}
 
-		_alpm_log(PM_LOG_DEBUG, "checking hashes for %s\n", entryname_orig);
-		_alpm_log(PM_LOG_DEBUG, "current:  %s\n", hash_local);
-		_alpm_log(PM_LOG_DEBUG, "new:      %s\n", hash_pkg);
-		_alpm_log(PM_LOG_DEBUG, "original: %s\n", hash_orig);
+		_alpm_log(handle, ALPM_LOG_DEBUG, "checking hashes for %s\n", entryname_orig);
+		_alpm_log(handle, ALPM_LOG_DEBUG, "current:  %s\n", hash_local);
+		_alpm_log(handle, ALPM_LOG_DEBUG, "new:      %s\n", hash_pkg);
+		_alpm_log(handle, ALPM_LOG_DEBUG, "original: %s\n", hash_orig);
 
 		if(!oldpkg) {
-			if(strcmp(hash_local, hash_pkg) != 0) {
+			if(hash_local && hash_pkg && strcmp(hash_local, hash_pkg) != 0) {
 				/* looks like we have a local file that has a different hash as the
 				 * file in the package, move it to a .pacorig */
 				char newpath[PATH_MAX];
@@ -339,22 +325,22 @@ static int extract_single_file(struct archive *archive,
 
 				/* move the existing file to the "pacorig" */
 				if(rename(filename, newpath)) {
-					_alpm_log(PM_LOG_ERROR, _("could not rename %s to %s (%s)\n"),
+					_alpm_log(handle, ALPM_LOG_ERROR, _("could not rename %s to %s (%s)\n"),
 							filename, newpath, strerror(errno));
-					alpm_logaction("error: could not rename %s to %s (%s)\n",
+					alpm_logaction(handle, "error: could not rename %s to %s (%s)\n",
 							filename, newpath, strerror(errno));
 					errors++;
 				} else {
 					/* rename the file we extracted to the real name */
 					if(rename(checkfile, filename)) {
-						_alpm_log(PM_LOG_ERROR, _("could not rename %s to %s (%s)\n"),
+						_alpm_log(handle, ALPM_LOG_ERROR, _("could not rename %s to %s (%s)\n"),
 								checkfile, filename, strerror(errno));
-						alpm_logaction("error: could not rename %s to %s (%s)\n",
+						alpm_logaction(handle, "error: could not rename %s to %s (%s)\n",
 								checkfile, filename, strerror(errno));
 						errors++;
 					} else {
-						_alpm_log(PM_LOG_WARNING, _("%s saved as %s\n"), filename, newpath);
-						alpm_logaction("warning: %s saved as %s\n", filename, newpath);
+						_alpm_log(handle, ALPM_LOG_WARNING, _("%s saved as %s\n"), filename, newpath);
+						alpm_logaction(handle, "warning: %s saved as %s\n", filename, newpath);
 					}
 				}
 			} else {
@@ -364,51 +350,51 @@ static int extract_single_file(struct archive *archive,
 		} else if(hash_orig) {
 			/* the fun part */
 
-			if(strcmp(hash_orig, hash_local) == 0) {
+			if(hash_local && strcmp(hash_orig, hash_local) == 0) {
 				/* installed file has NOT been changed by user */
-				if(strcmp(hash_orig, hash_pkg) != 0) {
-					_alpm_log(PM_LOG_DEBUG, "action: installing new file: %s\n",
+				if(hash_pkg && strcmp(hash_orig, hash_pkg) != 0) {
+					_alpm_log(handle, ALPM_LOG_DEBUG, "action: installing new file: %s\n",
 							entryname_orig);
 
 					if(rename(checkfile, filename)) {
-						_alpm_log(PM_LOG_ERROR, _("could not rename %s to %s (%s)\n"),
+						_alpm_log(handle, ALPM_LOG_ERROR, _("could not rename %s to %s (%s)\n"),
 								checkfile, filename, strerror(errno));
-						alpm_logaction("error: could not rename %s to %s (%s)\n",
+						alpm_logaction(handle, "error: could not rename %s to %s (%s)\n",
 								checkfile, filename, strerror(errno));
 						errors++;
 					}
 				} else {
-					/* there's no sense in installing the same file twice, install
-					 * ONLY is the original and package hashes differ */
-					_alpm_log(PM_LOG_DEBUG, "action: leaving existing file in place\n");
+					/* no sense in installing the same file twice, install
+					 * ONLY if the original and package hashes differ */
+					_alpm_log(handle, ALPM_LOG_DEBUG, "action: leaving existing file in place\n");
 					unlink(checkfile);
 				}
-			} else if(strcmp(hash_orig, hash_pkg) == 0) {
+			} else if(hash_pkg && strcmp(hash_orig, hash_pkg) == 0) {
 				/* originally installed file and new file are the same - this
 				 * implies the case above failed - i.e. the file was changed by a
 				 * user */
-				_alpm_log(PM_LOG_DEBUG, "action: leaving existing file in place\n");
+				_alpm_log(handle, ALPM_LOG_DEBUG, "action: leaving existing file in place\n");
 				unlink(checkfile);
-			} else if(strcmp(hash_local, hash_pkg) == 0) {
+			} else if(hash_local && hash_pkg && strcmp(hash_local, hash_pkg) == 0) {
 				/* this would be magical.  The above two cases failed, but the
 				 * user changes just so happened to make the new file exactly the
 				 * same as the one in the package... skip it */
-				_alpm_log(PM_LOG_DEBUG, "action: leaving existing file in place\n");
+				_alpm_log(handle, ALPM_LOG_DEBUG, "action: leaving existing file in place\n");
 				unlink(checkfile);
 			} else {
 				char newpath[PATH_MAX];
-				_alpm_log(PM_LOG_DEBUG, "action: keeping current file and installing"
+				_alpm_log(handle, ALPM_LOG_DEBUG, "action: keeping current file and installing"
 						" new one with .pacnew ending\n");
 				snprintf(newpath, PATH_MAX, "%s.pacnew", filename);
 				if(rename(checkfile, newpath)) {
-					_alpm_log(PM_LOG_ERROR, _("could not install %s as %s (%s)\n"),
+					_alpm_log(handle, ALPM_LOG_ERROR, _("could not install %s as %s (%s)\n"),
 							filename, newpath, strerror(errno));
-					alpm_logaction("error: could not install %s as %s (%s)\n",
+					alpm_logaction(handle, "error: could not install %s as %s (%s)\n",
 							filename, newpath, strerror(errno));
 				} else {
-					_alpm_log(PM_LOG_WARNING, _("%s installed as %s\n"),
+					_alpm_log(handle, ALPM_LOG_WARNING, _("%s installed as %s\n"),
 							filename, newpath);
-					alpm_logaction("warning: %s installed as %s\n",
+					alpm_logaction(handle, "warning: %s installed as %s\n",
 							filename, newpath);
 				}
 			}
@@ -416,126 +402,104 @@ static int extract_single_file(struct archive *archive,
 
 		FREE(hash_local);
 		FREE(hash_pkg);
-		FREE(hash_orig);
 	} else {
 		int ret;
 
 		/* we didn't need a backup */
 		if(notouch) {
 			/* change the path to a .pacnew extension */
-			_alpm_log(PM_LOG_DEBUG, "%s is in NoUpgrade -- skipping\n", filename);
-			_alpm_log(PM_LOG_WARNING, _("extracting %s as %s.pacnew\n"), filename, filename);
-			alpm_logaction("warning: extracting %s as %s.pacnew\n", filename, filename);
+			_alpm_log(handle, ALPM_LOG_DEBUG, "%s is in NoUpgrade -- skipping\n", filename);
+			_alpm_log(handle, ALPM_LOG_WARNING, _("extracting %s as %s.pacnew\n"), filename, filename);
+			alpm_logaction(handle, "warning: extracting %s as %s.pacnew\n", filename, filename);
 			strncat(filename, ".pacnew", PATH_MAX - strlen(filename));
 		} else {
-			_alpm_log(PM_LOG_DEBUG, "extracting %s\n", filename);
+			_alpm_log(handle, ALPM_LOG_DEBUG, "extracting %s\n", filename);
 		}
 
-		if(trans->flags & PM_TRANS_FLAG_FORCE) {
+		if(handle->trans->flags & ALPM_TRANS_FLAG_FORCE) {
 			/* if FORCE was used, unlink() each file (whether it's there
 			 * or not) before extracting. This prevents the old "Text file busy"
 			 * error that crops up if forcing a glibc or pacman upgrade. */
 			unlink(filename);
 		}
 
-		ret = perform_extraction(archive, entry, filename, entryname_orig);
+		ret = perform_extraction(handle, archive, entry, filename, entryname_orig);
 		if(ret == 1) {
 			/* error */
 			FREE(entryname_orig);
-			return(1);
+			return 1;
 		}
 
 		/* calculate an hash if this is in newpkg's backup */
-		alpm_list_t *b;
-		for(b = alpm_pkg_get_backup(newpkg); b; b = b->next) {
-			char *backup = NULL, *hash = NULL;
-			char *oldbackup = alpm_list_getdata(b);
-			/* length is tab char, null byte and MD5 (32 char) */
-			size_t backup_len = strlen(oldbackup) + 34;
-
-			if(!oldbackup || strcmp(oldbackup, entryname_orig) != 0) {
+		alpm_list_t *i;
+		for(i = alpm_pkg_get_backup(newpkg); i; i = i->next) {
+			alpm_backup_t *backup = i->data;
+			char *newhash;
+			if(!backup->name || strcmp(backup->name, entryname_orig) != 0) {
 				continue;
 			}
-			_alpm_log(PM_LOG_DEBUG, "appending backup entry for %s\n", filename);
-
-			hash = alpm_compute_md5sum(filename);
-			MALLOC(backup, backup_len, RET_ERR(PM_ERR_MEMORY, -1));
-
-			sprintf(backup, "%s\t%s", oldbackup, hash);
-			backup[backup_len-1] = '\0';
-			FREE(hash);
-			FREE(oldbackup);
-			b->data = backup;
+			_alpm_log(handle, ALPM_LOG_DEBUG, "appending backup entry for %s\n", entryname_orig);
+			newhash = alpm_compute_md5sum(filename);
+			FREE(backup->hash);
+			backup->hash = newhash;
 		}
 	}
 	FREE(entryname_orig);
-	return(errors);
+	return errors;
 }
 
-static int commit_single_pkg(pmpkg_t *newpkg, size_t pkg_current,
-		size_t pkg_count, pmtrans_t *trans, pmdb_t *db)
+static int commit_single_pkg(alpm_handle_t *handle, alpm_pkg_t *newpkg,
+		size_t pkg_current, size_t pkg_count)
 {
 	int i, ret = 0, errors = 0;
-	char scriptlet[PATH_MAX+1];
-	int is_upgrade = 0;
-	pmpkg_t *oldpkg = NULL;
+	int is_upgrade;
+	alpm_pkg_t *oldpkg = NULL;
+	alpm_db_t *db = handle->db_local;
+	alpm_trans_t *trans = handle->trans;
 
-	ALPM_LOG_FUNC;
-
-	ASSERT(trans != NULL, RET_ERR(PM_ERR_TRANS_NULL, -1));
-
-	snprintf(scriptlet, PATH_MAX, "%s%s-%s/install",
-			_alpm_db_path(db), alpm_pkg_get_name(newpkg),
-			alpm_pkg_get_version(newpkg));
+	ASSERT(trans != NULL, return -1);
 
 	/* see if this is an upgrade. if so, remove the old package first */
-	pmpkg_t *local = _alpm_db_get_pkgfromcache(db, newpkg->name);
+	alpm_pkg_t *local = _alpm_db_get_pkgfromcache(db, newpkg->name);
 	if(local) {
 		is_upgrade = 1;
 
 		/* we'll need to save some record for backup checks later */
-		oldpkg = _alpm_pkg_dup(local);
-		/* make sure all infos are loaded because the database entry
-		 * will be removed soon */
-		_alpm_local_db_read(oldpkg->origin_data.db, oldpkg, INFRQ_ALL);
-
-		EVENT(trans, PM_TRANS_EVT_UPGRADE_START, newpkg, oldpkg);
-		_alpm_log(PM_LOG_DEBUG, "upgrading package %s-%s\n",
-				newpkg->name, newpkg->version);
+		if(_alpm_pkg_dup(local, &oldpkg) == -1) {
+			ret = -1;
+			goto cleanup;
+		}
 
 		/* copy over the install reason */
-		newpkg->reason = alpm_pkg_get_reason(oldpkg);
+		newpkg->reason = alpm_pkg_get_reason(local);
 
-		/* pre_upgrade scriptlet */
-		if(alpm_pkg_has_scriptlet(newpkg) && !(trans->flags & PM_TRANS_FLAG_NOSCRIPTLET)) {
-			_alpm_runscriptlet(handle->root, newpkg->origin_data.file,
-					"pre_upgrade", newpkg->version, oldpkg->version, trans);
-		}
+		EVENT(handle, ALPM_EVENT_UPGRADE_START, newpkg, local);
 	} else {
 		is_upgrade = 0;
+		EVENT(handle, ALPM_EVENT_ADD_START, newpkg, NULL);
+	}
 
-		EVENT(trans, PM_TRANS_EVT_ADD_START, newpkg, NULL);
-		_alpm_log(PM_LOG_DEBUG, "adding package %s-%s\n",
-				newpkg->name, newpkg->version);
-
-		/* pre_install scriptlet */
-		if(alpm_pkg_has_scriptlet(newpkg) && !(trans->flags & PM_TRANS_FLAG_NOSCRIPTLET)) {
-			_alpm_runscriptlet(handle->root, newpkg->origin_data.file,
-					"pre_install", newpkg->version, NULL, trans);
-		}
+	_alpm_log(handle, ALPM_LOG_DEBUG, "%s package %s-%s\n",
+			is_upgrade ? "upgrading" : "adding", newpkg->name, newpkg->version);
+		/* pre_install/pre_upgrade scriptlet */
+	if(alpm_pkg_has_scriptlet(newpkg) &&
+			!(trans->flags & ALPM_TRANS_FLAG_NOSCRIPTLET)) {
+		const char *scriptlet_name = is_upgrade ? "pre_upgrade" : "pre_install";
+		_alpm_runscriptlet(handle, newpkg->origin_data.file,
+				scriptlet_name, newpkg->version, NULL, 1);
 	}
 
 	/* we override any pre-set reason if we have alldeps or allexplicit set */
-	if(trans->flags & PM_TRANS_FLAG_ALLDEPS) {
-		newpkg->reason = PM_PKG_REASON_DEPEND;
-	} else if(trans->flags & PM_TRANS_FLAG_ALLEXPLICIT) {
-		newpkg->reason = PM_PKG_REASON_EXPLICIT;
+	if(trans->flags & ALPM_TRANS_FLAG_ALLDEPS) {
+		newpkg->reason = ALPM_PKG_REASON_DEPEND;
+	} else if(trans->flags & ALPM_TRANS_FLAG_ALLEXPLICIT) {
+		newpkg->reason = ALPM_PKG_REASON_EXPLICIT;
 	}
 
 	if(oldpkg) {
 		/* set up fake remove transaction */
-		if(_alpm_upgraderemove_package(oldpkg, newpkg, trans) == -1) {
-			pm_errno = PM_ERR_TRANS_ABORT;
+		if(_alpm_remove_single_package(handle, oldpkg, newpkg, 0, 0) == -1) {
+			handle->pm_errno = ALPM_ERR_TRANS_ABORT;
 			ret = -1;
 			goto cleanup;
 		}
@@ -544,23 +508,22 @@ static int commit_single_pkg(pmpkg_t *newpkg, size_t pkg_current,
 	/* prepare directory for database entries so permission are correct after
 	   changelog/install script installation (FS#12263) */
 	if(_alpm_local_db_prepare(db, newpkg)) {
-		alpm_logaction("error: could not create database entry %s-%s\n",
-				alpm_pkg_get_name(newpkg), alpm_pkg_get_version(newpkg));
-		pm_errno = PM_ERR_DB_WRITE;
+		alpm_logaction(handle, "error: could not create database entry %s-%s\n",
+				newpkg->name, newpkg->version);
+		handle->pm_errno = ALPM_ERR_DB_WRITE;
 		ret = -1;
 		goto cleanup;
 	}
 
-	if(!(trans->flags & PM_TRANS_FLAG_DBONLY)) {
+	if(!(trans->flags & ALPM_TRANS_FLAG_DBONLY)) {
 		struct archive *archive;
 		struct archive_entry *entry;
-		char cwd[PATH_MAX] = "";
-		int restore_cwd = 0;
+		int cwdfd;
 
-		_alpm_log(PM_LOG_DEBUG, "extracting files\n");
+		_alpm_log(handle, ALPM_LOG_DEBUG, "extracting files\n");
 
-		if ((archive = archive_read_new()) == NULL) {
-			pm_errno = PM_ERR_LIBARCHIVE;
+		if((archive = archive_read_new()) == NULL) {
+			handle->pm_errno = ALPM_ERR_LIBARCHIVE;
 			ret = -1;
 			goto cleanup;
 		}
@@ -568,35 +531,37 @@ static int commit_single_pkg(pmpkg_t *newpkg, size_t pkg_current,
 		archive_read_support_compression_all(archive);
 		archive_read_support_format_all(archive);
 
-		_alpm_log(PM_LOG_DEBUG, "archive: %s\n", newpkg->origin_data.file);
+		_alpm_log(handle, ALPM_LOG_DEBUG, "archive: %s\n", newpkg->origin_data.file);
 		if(archive_read_open_filename(archive, newpkg->origin_data.file,
 					ARCHIVE_DEFAULT_BYTES_PER_BLOCK) != ARCHIVE_OK) {
-			pm_errno = PM_ERR_PKG_OPEN;
+			handle->pm_errno = ALPM_ERR_PKG_OPEN;
 			ret = -1;
 			goto cleanup;
 		}
 
 		/* save the cwd so we can restore it later */
-		if(getcwd(cwd, PATH_MAX) == NULL) {
-			_alpm_log(PM_LOG_ERROR, _("could not get current working directory\n"));
-		} else {
-			restore_cwd = 1;
+		do {
+			cwdfd = open(".", O_RDONLY);
+		} while(cwdfd == -1 && errno == EINTR);
+		if(cwdfd < 0) {
+			_alpm_log(handle, ALPM_LOG_ERROR, _("could not get current working directory\n"));
 		}
 
 		/* libarchive requires this for extracting hard links */
 		if(chdir(handle->root) != 0) {
-			_alpm_log(PM_LOG_ERROR, _("could not change directory to %s (%s)\n"), handle->root, strerror(errno));
+			_alpm_log(handle, ALPM_LOG_ERROR, _("could not change directory to %s (%s)\n"),
+					handle->root, strerror(errno));
 			ret = -1;
 			goto cleanup;
 		}
 
 		/* call PROGRESS once with 0 percent, as we sort-of skip that here */
 		if(is_upgrade) {
-			PROGRESS(trans, PM_TRANS_PROGRESS_UPGRADE_START,
-					alpm_pkg_get_name(newpkg), 0, pkg_count, pkg_current);
+			PROGRESS(handle, ALPM_PROGRESS_UPGRADE_START,
+					newpkg->name, 0, pkg_count, pkg_current);
 		} else {
-			PROGRESS(trans, PM_TRANS_PROGRESS_ADD_START,
-					alpm_pkg_get_name(newpkg), 0, pkg_count, pkg_current);
+			PROGRESS(handle, ALPM_PROGRESS_ADD_START,
+					newpkg->name, 0, pkg_count, pkg_current);
 		}
 
 		for(i = 0; archive_read_next_header(archive, &entry) == ARCHIVE_OK; i++) {
@@ -608,9 +573,6 @@ static int commit_single_pkg(pmpkg_t *newpkg, size_t pkg_current,
 				 * (missing metadata sizes) */
 				int64_t pos = archive_position_compressed(archive);
 				percent = (pos * 100) / newpkg->size;
-				_alpm_log(PM_LOG_DEBUG, "decompression progress: "
-						"%d%% (%"PRId64" / %jd)\n",
-						percent, pos, (intmax_t)newpkg->size);
 				if(percent >= 100) {
 					percent = 100;
 				}
@@ -619,37 +581,38 @@ static int commit_single_pkg(pmpkg_t *newpkg, size_t pkg_current,
 			}
 
 			if(is_upgrade) {
-				PROGRESS(trans, PM_TRANS_PROGRESS_UPGRADE_START,
-						alpm_pkg_get_name(newpkg), percent, pkg_count,
-						pkg_current);
+				PROGRESS(handle, ALPM_PROGRESS_UPGRADE_START,
+						newpkg->name, percent, pkg_count, pkg_current);
 			} else {
-				PROGRESS(trans, PM_TRANS_PROGRESS_ADD_START,
-						alpm_pkg_get_name(newpkg), percent, pkg_count,
-						pkg_current);
+				PROGRESS(handle, ALPM_PROGRESS_ADD_START,
+						newpkg->name, percent, pkg_count, pkg_current);
 			}
 
 			/* extract the next file from the archive */
-			errors += extract_single_file(archive, entry, newpkg, oldpkg,
-					trans, db);
+			errors += extract_single_file(handle, archive, entry, newpkg, oldpkg);
 		}
 		archive_read_finish(archive);
 
 		/* restore the old cwd if we have it */
-		if(restore_cwd && chdir(cwd) != 0) {
-			_alpm_log(PM_LOG_ERROR, _("could not change directory to %s (%s)\n"), cwd, strerror(errno));
+		if(cwdfd >= 0) {
+			if(fchdir(cwdfd) != 0) {
+				_alpm_log(handle, ALPM_LOG_ERROR,
+						_("could not restore working directory (%s)\n"), strerror(errno));
+			}
+			close(cwdfd);
 		}
 
 		if(errors) {
 			ret = -1;
 			if(is_upgrade) {
-				_alpm_log(PM_LOG_ERROR, _("problem occurred while upgrading %s\n"),
+				_alpm_log(handle, ALPM_LOG_ERROR, _("problem occurred while upgrading %s\n"),
 						newpkg->name);
-				alpm_logaction("error: problem occurred while upgrading %s\n",
+				alpm_logaction(handle, "error: problem occurred while upgrading %s\n",
 						newpkg->name);
 			} else {
-				_alpm_log(PM_LOG_ERROR, _("problem occurred while installing %s\n"),
+				_alpm_log(handle, ALPM_LOG_ERROR, _("problem occurred while installing %s\n"),
 						newpkg->name);
-				alpm_logaction("error: problem occurred while installing %s\n",
+				alpm_logaction(handle, "error: problem occurred while installing %s\n",
 						newpkg->name);
 			}
 		}
@@ -658,69 +621,63 @@ static int commit_single_pkg(pmpkg_t *newpkg, size_t pkg_current,
 	/* make an install date (in UTC) */
 	newpkg->installdate = time(NULL);
 
-	_alpm_log(PM_LOG_DEBUG, "updating database\n");
-	_alpm_log(PM_LOG_DEBUG, "adding database entry '%s'\n", newpkg->name);
+	_alpm_log(handle, ALPM_LOG_DEBUG, "updating database\n");
+	_alpm_log(handle, ALPM_LOG_DEBUG, "adding database entry '%s'\n", newpkg->name);
 
 	if(_alpm_local_db_write(db, newpkg, INFRQ_ALL)) {
-		_alpm_log(PM_LOG_ERROR, _("could not update database entry %s-%s\n"),
-				alpm_pkg_get_name(newpkg), alpm_pkg_get_version(newpkg));
-		alpm_logaction("error: could not update database entry %s-%s\n",
-				alpm_pkg_get_name(newpkg), alpm_pkg_get_version(newpkg));
-		pm_errno = PM_ERR_DB_WRITE;
+		_alpm_log(handle, ALPM_LOG_ERROR, _("could not update database entry %s-%s\n"),
+				newpkg->name, newpkg->version);
+		alpm_logaction(handle, "error: could not update database entry %s-%s\n",
+				newpkg->name, newpkg->version);
+		handle->pm_errno = ALPM_ERR_DB_WRITE;
 		ret = -1;
 		goto cleanup;
 	}
 
 	if(_alpm_db_add_pkgincache(db, newpkg) == -1) {
-		_alpm_log(PM_LOG_ERROR, _("could not add entry '%s' in cache\n"),
-				alpm_pkg_get_name(newpkg));
+		_alpm_log(handle, ALPM_LOG_ERROR, _("could not add entry '%s' in cache\n"),
+				newpkg->name);
 	}
 
 	if(is_upgrade) {
-		PROGRESS(trans, PM_TRANS_PROGRESS_UPGRADE_START,
-				alpm_pkg_get_name(newpkg), 100, pkg_count, pkg_current);
+		PROGRESS(handle, ALPM_PROGRESS_UPGRADE_START,
+				newpkg->name, 100, pkg_count, pkg_current);
 	} else {
-		PROGRESS(trans, PM_TRANS_PROGRESS_ADD_START,
-				alpm_pkg_get_name(newpkg), 100, pkg_count, pkg_current);
+		PROGRESS(handle, ALPM_PROGRESS_ADD_START,
+				newpkg->name, 100, pkg_count, pkg_current);
 	}
 
 	/* run the post-install script if it exists  */
 	if(alpm_pkg_has_scriptlet(newpkg)
-			&& !(trans->flags & PM_TRANS_FLAG_NOSCRIPTLET)) {
-		if(is_upgrade) {
-			_alpm_runscriptlet(handle->root, scriptlet, "post_upgrade",
-					alpm_pkg_get_version(newpkg),
-					oldpkg ? alpm_pkg_get_version(oldpkg) : NULL, trans);
-		} else {
-			_alpm_runscriptlet(handle->root, scriptlet, "post_install",
-					alpm_pkg_get_version(newpkg), NULL, trans);
-		}
+			&& !(trans->flags & ALPM_TRANS_FLAG_NOSCRIPTLET)) {
+		char *scriptlet = _alpm_local_db_pkgpath(db, newpkg, "install");
+		const char *scriptlet_name = is_upgrade ? "post_upgrade" : "post_install";
+
+		_alpm_runscriptlet(handle, scriptlet, scriptlet_name,
+				newpkg->version, oldpkg ? oldpkg->version : NULL, 0);
+		free(scriptlet);
 	}
 
 	if(is_upgrade) {
-		EVENT(trans, PM_TRANS_EVT_UPGRADE_DONE, newpkg, oldpkg);
+		EVENT(handle, ALPM_EVENT_UPGRADE_DONE, newpkg, oldpkg);
 	} else {
-		EVENT(trans, PM_TRANS_EVT_ADD_DONE, newpkg, oldpkg);
+		EVENT(handle, ALPM_EVENT_ADD_DONE, newpkg, oldpkg);
 	}
 
 cleanup:
 	_alpm_pkg_free(oldpkg);
-	return(ret);
+	return ret;
 }
 
-int _alpm_upgrade_packages(pmtrans_t *trans, pmdb_t *db)
+int _alpm_upgrade_packages(alpm_handle_t *handle)
 {
 	size_t pkg_count, pkg_current;
 	int skip_ldconfig = 0, ret = 0;
 	alpm_list_t *targ;
-
-	ALPM_LOG_FUNC;
-
-	ASSERT(trans != NULL, RET_ERR(PM_ERR_TRANS_NULL, -1));
-	ASSERT(db != NULL, RET_ERR(PM_ERR_DB_NULL, -1));
+	alpm_trans_t *trans = handle->trans;
 
 	if(trans->add == NULL) {
-		return(0);
+		return 0;
 	}
 
 	pkg_count = alpm_list_count(trans->add);
@@ -728,15 +685,16 @@ int _alpm_upgrade_packages(pmtrans_t *trans, pmdb_t *db)
 
 	/* loop through our package list adding/upgrading one at a time */
 	for(targ = trans->add; targ; targ = targ->next) {
+		alpm_pkg_t *newpkg = targ->data;
+
 		if(handle->trans->state == STATE_INTERRUPTED) {
-			return(ret);
+			return ret;
 		}
 
-		pmpkg_t *newpkg = (pmpkg_t *)targ->data;
-		if(commit_single_pkg(newpkg, pkg_current, pkg_count, trans, db)) {
+		if(commit_single_pkg(handle, newpkg, pkg_current, pkg_count)) {
 			/* something screwed up on the commit, abort the trans */
 			trans->state = STATE_INTERRUPTED;
-			pm_errno = PM_ERR_TRANS_ABORT;
+			handle->pm_errno = ALPM_ERR_TRANS_ABORT;
 			/* running ldconfig at this point could possibly screw system */
 			skip_ldconfig = 1;
 			ret = -1;
@@ -747,10 +705,10 @@ int _alpm_upgrade_packages(pmtrans_t *trans, pmdb_t *db)
 
 	if(!skip_ldconfig) {
 		/* run ldconfig if it exists */
-		_alpm_ldconfig(handle->root);
+		_alpm_ldconfig(handle);
 	}
 
-	return(ret);
+	return ret;
 }
 
 /* vim: set ts=2 sw=2 noet: */
