@@ -35,31 +35,30 @@
 #include "package.h"
 #include "db.h"
 #include "handle.h"
+#include "trans.h"
 
-void _alpm_dep_free(pmdepend_t *dep)
+void _alpm_dep_free(alpm_depend_t *dep)
 {
 	FREE(dep->name);
 	FREE(dep->version);
 	FREE(dep);
 }
 
-static pmdepmissing_t *depmiss_new(const char *target, pmdepend_t *dep,
+static alpm_depmissing_t *depmiss_new(const char *target, alpm_depend_t *dep,
 		const char *causingpkg)
 {
-	pmdepmissing_t *miss;
+	alpm_depmissing_t *miss;
 
-	ALPM_LOG_FUNC;
+	MALLOC(miss, sizeof(alpm_depmissing_t), return NULL);
 
-	MALLOC(miss, sizeof(pmdepmissing_t), RET_ERR(PM_ERR_MEMORY, NULL));
-
-	STRDUP(miss->target, target, RET_ERR(PM_ERR_MEMORY, NULL));
+	STRDUP(miss->target, target, return NULL);
 	miss->depend = _alpm_dep_dup(dep);
-	STRDUP(miss->causingpkg, causingpkg, RET_ERR(PM_ERR_MEMORY, NULL));
+	STRDUP(miss->causingpkg, causingpkg, return NULL);
 
-	return(miss);
+	return miss;
 }
 
-void _alpm_depmiss_free(pmdepmissing_t *miss)
+void _alpm_depmiss_free(alpm_depmissing_t *miss)
 {
 	_alpm_dep_free(miss->depend);
 	FREE(miss->target);
@@ -68,18 +67,18 @@ void _alpm_depmiss_free(pmdepmissing_t *miss)
 }
 
 /* Does pkg1 depend on pkg2, ie. does pkg2 satisfy a dependency of pkg1? */
-static int _alpm_dep_edge(pmpkg_t *pkg1, pmpkg_t *pkg2)
+static int _alpm_dep_edge(alpm_pkg_t *pkg1, alpm_pkg_t *pkg2)
 {
 	alpm_list_t *i;
 	for(i = alpm_pkg_get_depends(pkg1); i; i = i->next) {
 		if(_alpm_depcmp(pkg2, i->data)) {
-			return(1);
+			return 1;
 		}
 	}
-	return(0);
+	return 0;
 }
 
-/* Convert a list of pmpkg_t * to a graph structure,
+/* Convert a list of alpm_pkg_t * to a graph structure,
  * with a edge for each dependency.
  * Returns a list of vertices (one vertex = one package)
  * (used by alpm_sortbydeps)
@@ -90,19 +89,19 @@ static alpm_list_t *dep_graph_init(alpm_list_t *targets)
 	alpm_list_t *vertices = NULL;
 	/* We create the vertices */
 	for(i = targets; i; i = i->next) {
-		pmgraph_t *vertex = _alpm_graph_new();
+		alpm_graph_t *vertex = _alpm_graph_new();
 		vertex->data = (void *)i->data;
 		vertices = alpm_list_add(vertices, vertex);
 	}
 
 	/* We compute the edges */
 	for(i = vertices; i; i = i->next) {
-		pmgraph_t *vertex_i = i->data;
-		pmpkg_t *p_i = vertex_i->data;
+		alpm_graph_t *vertex_i = i->data;
+		alpm_pkg_t *p_i = vertex_i->data;
 		/* TODO this should be somehow combined with alpm_checkdeps */
 		for(j = vertices; j; j = j->next) {
-			pmgraph_t *vertex_j = j->data;
-			pmpkg_t *p_j = vertex_j->data;
+			alpm_graph_t *vertex_j = j->data;
+			alpm_pkg_t *p_j = vertex_j->data;
 			if(_alpm_dep_edge(p_i, p_j)) {
 				vertex_i->children =
 					alpm_list_add(vertex_i->children, vertex_j);
@@ -110,7 +109,7 @@ static alpm_list_t *dep_graph_init(alpm_list_t *targets)
 		}
 		vertex_i->childptr = vertex_i->children;
 	}
-	return(vertices);
+	return vertices;
 }
 
 /* Re-order a list of target packages with respect to their dependencies.
@@ -127,20 +126,19 @@ static alpm_list_t *dep_graph_init(alpm_list_t *targets)
  * This function returns the new alpm_list_t* target list.
  *
  */
-alpm_list_t *_alpm_sortbydeps(alpm_list_t *targets, int reverse)
+alpm_list_t *_alpm_sortbydeps(alpm_handle_t *handle,
+		alpm_list_t *targets, int reverse)
 {
 	alpm_list_t *newtargs = NULL;
 	alpm_list_t *vertices = NULL;
 	alpm_list_t *vptr;
-	pmgraph_t *vertex;
-
-	ALPM_LOG_FUNC;
+	alpm_graph_t *vertex;
 
 	if(targets == NULL) {
-		return(NULL);
+		return NULL;
 	}
 
-	_alpm_log(PM_LOG_DEBUG, "started sorting dependencies\n");
+	_alpm_log(handle, ALPM_LOG_DEBUG, "started sorting dependencies\n");
 
 	vertices = dep_graph_init(targets);
 
@@ -151,25 +149,25 @@ alpm_list_t *_alpm_sortbydeps(alpm_list_t *targets, int reverse)
 		vertex->state = -1;
 		int found = 0;
 		while(vertex->childptr && !found) {
-			pmgraph_t *nextchild = vertex->childptr->data;
+			alpm_graph_t *nextchild = vertex->childptr->data;
 			vertex->childptr = vertex->childptr->next;
-			if (nextchild->state == 0) {
+			if(nextchild->state == 0) {
 				found = 1;
 				nextchild->parent = vertex;
 				vertex = nextchild;
 			}
 			else if(nextchild->state == -1) {
-				pmpkg_t *vertexpkg = vertex->data;
-				pmpkg_t *childpkg = nextchild->data;
+				alpm_pkg_t *vertexpkg = vertex->data;
+				alpm_pkg_t *childpkg = nextchild->data;
 				const char *message;
 
-				_alpm_log(PM_LOG_WARNING, _("dependency cycle detected:\n"));
+				_alpm_log(handle, ALPM_LOG_WARNING, _("dependency cycle detected:\n"));
 				if(reverse) {
 					message =_("%s will be removed after its %s dependency\n");
 				} else {
 					message =_("%s will be installed before its %s dependency\n");
 				}
-				_alpm_log(PM_LOG_WARNING, message, vertexpkg->name, childpkg->name);
+				_alpm_log(handle, ALPM_LOG_WARNING, message, vertexpkg->name, childpkg->name);
 			}
 		}
 		if(!found) {
@@ -181,14 +179,14 @@ alpm_list_t *_alpm_sortbydeps(alpm_list_t *targets, int reverse)
 				vptr = vptr->next;
 				while(vptr) {
 					vertex = vptr->data;
-					if (vertex->state == 0) break;
+					if(vertex->state == 0) break;
 					vptr = vptr->next;
 				}
 			}
 		}
 	}
 
-	_alpm_log(PM_LOG_DEBUG, "sorting dependencies finished\n");
+	_alpm_log(handle, ALPM_LOG_DEBUG, "sorting dependencies finished\n");
 
 	if(reverse) {
 		/* reverse the order */
@@ -201,111 +199,114 @@ alpm_list_t *_alpm_sortbydeps(alpm_list_t *targets, int reverse)
 	alpm_list_free_inner(vertices, _alpm_graph_free);
 	alpm_list_free(vertices);
 
-	return(newtargs);
+	return newtargs;
 }
 
-static int no_dep_version(void)
+static int no_dep_version(alpm_handle_t *handle)
 {
-	int flags = alpm_trans_get_flags();
-	return flags != -1 && (flags & PM_TRANS_FLAG_NODEPVERSION);
+	int flags = alpm_trans_get_flags(handle);
+	return flags != -1 && (flags & ALPM_TRANS_FLAG_NODEPVERSION);
 }
 
-static pmdepend_t *filtered_depend(pmdepend_t *dep, int nodepversion)
+static alpm_depend_t *filtered_depend(alpm_depend_t *dep, int nodepversion)
 {
 	if(nodepversion) {
-		pmdepend_t *newdep = _alpm_dep_dup(dep);
-		ASSERT(newdep, return(dep));
-		newdep->mod = PM_DEP_MOD_ANY;
+		alpm_depend_t *newdep = _alpm_dep_dup(dep);
+		ASSERT(newdep, return dep);
+		newdep->mod = ALPM_DEP_MOD_ANY;
 		dep = newdep;
 	}
 	return dep;
 }
 
-static void release_filtered_depend(pmdepend_t *dep, int nodepversion)
+static void release_filtered_depend(alpm_depend_t *dep, int nodepversion)
 {
 	if(nodepversion) {
 		free(dep);
 	}
 }
 
-static pmpkg_t *find_dep_satisfier(alpm_list_t *pkgs, pmdepend_t *dep)
+static alpm_pkg_t *find_dep_satisfier(alpm_list_t *pkgs, alpm_depend_t *dep)
 {
 	alpm_list_t *i;
 
-	for(i = pkgs; i; i = alpm_list_next(i)) {
-		pmpkg_t *pkg = i->data;
+	for(i = pkgs; i; i = i->next) {
+		alpm_pkg_t *pkg = i->data;
 		if(_alpm_depcmp(pkg, dep)) {
-			return(pkg);
+			return pkg;
 		}
 	}
-	return(NULL);
+	return NULL;
 }
 
 /** Find a package satisfying a specified dependency.
  * The dependency can include versions with depmod operators.
- * @param pkgs an alpm_list_t* of pmpkg_t where the satisfier will be searched
+ * @param pkgs an alpm_list_t* of alpm_pkg_t where the satisfier will be searched
  * @param depstring package or provision name, versioned or not
- * @return a pmpkg_t* satisfying depstring
+ * @return a alpm_pkg_t* satisfying depstring
  */
-pmpkg_t SYMEXPORT *alpm_find_satisfier(alpm_list_t *pkgs, const char *depstring)
+alpm_pkg_t SYMEXPORT *alpm_find_satisfier(alpm_list_t *pkgs, const char *depstring)
 {
-	pmdepend_t *dep = _alpm_splitdep(depstring);
-	pmpkg_t *pkg = find_dep_satisfier(pkgs, dep);
+	alpm_depend_t *dep = _alpm_splitdep(depstring);
+	if(!dep) {
+		return NULL;
+	}
+	alpm_pkg_t *pkg = find_dep_satisfier(pkgs, dep);
 	_alpm_dep_free(dep);
-	return(pkg);
+	return pkg;
 }
 
 /** Checks dependencies and returns missing ones in a list.
  * Dependencies can include versions with depmod operators.
+ * @param handle the context handle
  * @param pkglist the list of local packages
- * @param reversedeps handles the backward dependencies
  * @param remove an alpm_list_t* of packages to be removed
  * @param upgrade an alpm_list_t* of packages to be upgraded (remove-then-upgrade)
- * @return an alpm_list_t* of pmdepmissing_t pointers.
+ * @param reversedeps handles the backward dependencies
+ * @return an alpm_list_t* of alpm_depmissing_t pointers.
  */
-alpm_list_t SYMEXPORT *alpm_checkdeps(alpm_list_t *pkglist, int reversedeps,
-		alpm_list_t *remove, alpm_list_t *upgrade)
+alpm_list_t SYMEXPORT *alpm_checkdeps(alpm_handle_t *handle,
+		alpm_list_t *pkglist, alpm_list_t *remove, alpm_list_t *upgrade,
+		int reversedeps)
 {
 	alpm_list_t *i, *j;
-	alpm_list_t *targets, *dblist = NULL, *modified = NULL;
+	alpm_list_t *dblist = NULL, *modified = NULL;
 	alpm_list_t *baddeps = NULL;
 	int nodepversion;
 
-	ALPM_LOG_FUNC;
+	CHECK_HANDLE(handle, return NULL);
 
-	targets = alpm_list_join(alpm_list_copy(remove), alpm_list_copy(upgrade));
 	for(i = pkglist; i; i = i->next) {
-		pmpkg_t *pkg = i->data;
-		if(_alpm_pkg_find(targets, pkg->name)) {
+		alpm_pkg_t *pkg = i->data;
+		if(_alpm_pkg_find(remove, pkg->name) || _alpm_pkg_find(upgrade, pkg->name)) {
 			modified = alpm_list_add(modified, pkg);
 		} else {
 			dblist = alpm_list_add(dblist, pkg);
 		}
 	}
-	alpm_list_free(targets);
 
-	nodepversion = no_dep_version();
+	nodepversion = no_dep_version(handle);
 
 	/* look for unsatisfied dependencies of the upgrade list */
 	for(i = upgrade; i; i = i->next) {
-		pmpkg_t *tp = i->data;
-		_alpm_log(PM_LOG_DEBUG, "checkdeps: package %s-%s\n",
-				alpm_pkg_get_name(tp), alpm_pkg_get_version(tp));
+		alpm_pkg_t *tp = i->data;
+		_alpm_log(handle, ALPM_LOG_DEBUG, "checkdeps: package %s-%s\n",
+				tp->name, tp->version);
 
 		for(j = alpm_pkg_get_depends(tp); j; j = j->next) {
-			pmdepend_t *depend = j->data;
+			alpm_depend_t *depend = j->data;
 			depend = filtered_depend(depend, nodepversion);
 			/* 1. we check the upgrade list */
 			/* 2. we check database for untouched satisfying packages */
 			if(!find_dep_satisfier(upgrade, depend) &&
-			   !find_dep_satisfier(dblist, depend)) {
+					!find_dep_satisfier(dblist, depend)) {
 				/* Unsatisfied dependency in the upgrade list */
-				pmdepmissing_t *miss;
+				alpm_depmissing_t *miss;
 				char *missdepstring = alpm_dep_compute_string(depend);
-				_alpm_log(PM_LOG_DEBUG, "checkdeps: missing dependency '%s' for package '%s'\n",
-						missdepstring, alpm_pkg_get_name(tp));
+				_alpm_log(handle, ALPM_LOG_DEBUG, "checkdeps: missing dependency '%s' for package '%s'\n",
+						missdepstring, tp->name);
 				free(missdepstring);
-				miss = depmiss_new(alpm_pkg_get_name(tp), depend, NULL);
+				miss = depmiss_new(tp->name, depend, NULL);
 				baddeps = alpm_list_add(baddeps, miss);
 			}
 			release_filtered_depend(depend, nodepversion);
@@ -316,23 +317,23 @@ alpm_list_t SYMEXPORT *alpm_checkdeps(alpm_list_t *pkglist, int reversedeps,
 		/* reversedeps handles the backwards dependencies, ie,
 		 * the packages listed in the requiredby field. */
 		for(i = dblist; i; i = i->next) {
-			pmpkg_t *lp = i->data;
+			alpm_pkg_t *lp = i->data;
 			for(j = alpm_pkg_get_depends(lp); j; j = j->next) {
-				pmdepend_t *depend = j->data;
+				alpm_depend_t *depend = j->data;
 				depend = filtered_depend(depend, nodepversion);
-				pmpkg_t *causingpkg = find_dep_satisfier(modified, depend);
+				alpm_pkg_t *causingpkg = find_dep_satisfier(modified, depend);
 				/* we won't break this depend, if it is already broken, we ignore it */
 				/* 1. check upgrade list for satisfiers */
 				/* 2. check dblist for satisfiers */
 				if(causingpkg &&
 				   !find_dep_satisfier(upgrade, depend) &&
 				   !find_dep_satisfier(dblist, depend)) {
-					pmdepmissing_t *miss;
+					alpm_depmissing_t *miss;
 					char *missdepstring = alpm_dep_compute_string(depend);
-					_alpm_log(PM_LOG_DEBUG, "checkdeps: transaction would break '%s' dependency of '%s'\n",
-							missdepstring, alpm_pkg_get_name(lp));
+					_alpm_log(handle, ALPM_LOG_DEBUG, "checkdeps: transaction would break '%s' dependency of '%s'\n",
+							missdepstring, lp->name);
 					free(missdepstring);
-					miss = depmiss_new(lp->name, depend, alpm_pkg_get_name(causingpkg));
+					miss = depmiss_new(lp->name, depend, causingpkg->name);
 					baddeps = alpm_list_add(baddeps, miss);
 				}
 				release_filtered_depend(depend, nodepversion);
@@ -343,149 +344,156 @@ alpm_list_t SYMEXPORT *alpm_checkdeps(alpm_list_t *pkglist, int reversedeps,
 	alpm_list_free(modified);
 	alpm_list_free(dblist);
 
-	return(baddeps);
+	return baddeps;
 }
 
-static int dep_vercmp(const char *version1, pmdepmod_t mod,
+static int dep_vercmp(const char *version1, alpm_depmod_t mod,
 		const char *version2)
 {
 	int equal = 0;
 
-	if(mod == PM_DEP_MOD_ANY) {
+	if(mod == ALPM_DEP_MOD_ANY) {
 		equal = 1;
 	} else {
 		int cmp = alpm_pkg_vercmp(version1, version2);
 		switch(mod) {
-			case PM_DEP_MOD_EQ: equal = (cmp == 0); break;
-			case PM_DEP_MOD_GE: equal = (cmp >= 0); break;
-			case PM_DEP_MOD_LE: equal = (cmp <= 0); break;
-			case PM_DEP_MOD_LT: equal = (cmp < 0); break;
-			case PM_DEP_MOD_GT: equal = (cmp > 0); break;
+			case ALPM_DEP_MOD_EQ: equal = (cmp == 0); break;
+			case ALPM_DEP_MOD_GE: equal = (cmp >= 0); break;
+			case ALPM_DEP_MOD_LE: equal = (cmp <= 0); break;
+			case ALPM_DEP_MOD_LT: equal = (cmp < 0); break;
+			case ALPM_DEP_MOD_GT: equal = (cmp > 0); break;
 			default: equal = 1; break;
 		}
 	}
-	return(equal);
+	return equal;
 }
 
-int _alpm_depcmp(pmpkg_t *pkg, pmdepend_t *dep)
+int _alpm_depcmp_literal(alpm_pkg_t *pkg, alpm_depend_t *dep)
+{
+	if(pkg->name_hash != dep->name_hash
+			|| strcmp(pkg->name, dep->name) != 0) {
+		/* skip more expensive checks */
+		return 0;
+	}
+	return dep_vercmp(pkg->version, dep->mod, dep->version);
+}
+
+int _alpm_depcmp(alpm_pkg_t *pkg, alpm_depend_t *dep)
 {
 	alpm_list_t *i;
-	int satisfy = 0;
+	int satisfy = _alpm_depcmp_literal(pkg, dep);
 
-	/* check (pkg->name, pkg->version) */
-	if(pkg->name_hash && dep->name_hash
-			&& pkg->name_hash != dep->name_hash) {
-		/* skip more expensive checks */
-	} else {
-		satisfy = (strcmp(pkg->name, dep->name) == 0
-				&& dep_vercmp(pkg->version, dep->mod, dep->version));
-		if(satisfy) {
-			return(satisfy);
-		}
+	if(satisfy) {
+		return satisfy;
 	}
 
-	/* check provisions, format : "name=version" */
+	/* check provisions, name and version if available */
 	for(i = alpm_pkg_get_provides(pkg); i && !satisfy; i = i->next) {
-		const char *provision = i->data;
-		const char *provver = strchr(provision, '=');
+		alpm_depend_t *provision = i->data;
 
-		if(provver == NULL) { /* no provision version */
-			satisfy = (dep->mod == PM_DEP_MOD_ANY
-					&& strcmp(provision, dep->name) == 0);
-		} else {
-			/* This is a bit tricker than the old code for performance reasons. To
-			 * prevent the need to copy and duplicate strings, strncmp only the name
-			 * portion if they are the same length, since there is a version and
-			 * operator in play here. Cast is to silence sign conversion warning;
-			 * we know provver >= provision if we are here. */
-			size_t namelen = (size_t)(provver - provision);
-			provver += 1;
-			satisfy = (strlen(dep->name) == namelen
-					&& strncmp(provision, dep->name, namelen) == 0
-					&& dep_vercmp(provver, dep->mod, dep->version));
+		if(dep->mod == ALPM_DEP_MOD_ANY) {
+			/* any version will satisfy the requirement */
+			satisfy = (provision->name_hash == dep->name_hash
+					&& strcmp(provision->name, dep->name) == 0);
+		} else if (provision->mod == ALPM_DEP_MOD_EQ) {
+			/* provision specifies a version, so try it out */
+			satisfy = (provision->name_hash == dep->name_hash
+					&& strcmp(provision->name, dep->name) == 0
+					&& dep_vercmp(provision->version, dep->mod, dep->version));
 		}
 	}
 
-	return(satisfy);
+	return satisfy;
 }
 
-pmdepend_t *_alpm_splitdep(const char *depstring)
+alpm_depend_t *_alpm_splitdep(const char *depstring)
 {
-	pmdepend_t *depend;
-	const char *ptr, *version = NULL;
+	alpm_depend_t *depend;
+	const char *ptr, *version;
+	size_t deplen;
 
 	if(depstring == NULL) {
-		return(NULL);
+		return NULL;
 	}
 
-	CALLOC(depend, 1, sizeof(pmdepend_t), RET_ERR(PM_ERR_MEMORY, NULL));
+	MALLOC(depend, sizeof(alpm_depend_t), return NULL);
+	deplen = strlen(depstring);
 
 	/* Find a version comparator if one exists. If it does, set the type and
 	 * increment the ptr accordingly so we can copy the right strings. */
-	if((ptr = strstr(depstring, ">="))) {
-		depend->mod = PM_DEP_MOD_GE;
-		version = ptr + 2;
-	} else if((ptr = strstr(depstring, "<="))) {
-		depend->mod = PM_DEP_MOD_LE;
-		version = ptr + 2;
-	} else if((ptr = strstr(depstring, "="))) {
+	if((ptr = memchr(depstring, '<', deplen))) {
+		if(ptr[1] == '=') {
+			depend->mod = ALPM_DEP_MOD_LE;
+			version = ptr + 2;
+		} else {
+			depend->mod = ALPM_DEP_MOD_LT;
+			version = ptr + 1;
+		}
+	} else if((ptr = memchr(depstring, '>', deplen))) {
+		if(ptr[1] == '=') {
+			depend->mod = ALPM_DEP_MOD_GE;
+			version = ptr + 2;
+		} else {
+			depend->mod = ALPM_DEP_MOD_GT;
+			version = ptr + 1;
+		}
+	} else if((ptr = memchr(depstring, '=', deplen))) {
 		/* Note: we must do =,<,> checks after <=, >= checks */
-		depend->mod = PM_DEP_MOD_EQ;
-		version = ptr + 1;
-	} else if((ptr = strstr(depstring, "<"))) {
-		depend->mod = PM_DEP_MOD_LT;
-		version = ptr + 1;
-	} else if((ptr = strstr(depstring, ">"))) {
-		depend->mod = PM_DEP_MOD_GT;
+		depend->mod = ALPM_DEP_MOD_EQ;
 		version = ptr + 1;
 	} else {
-		/* no version specified, leave version and ptr NULL */
-		depend->mod = PM_DEP_MOD_ANY;
+		/* no version specified, leave ptr NULL and set version to NULL */
+		depend->mod = ALPM_DEP_MOD_ANY;
+		depend->version = NULL;
+		version = NULL;
 	}
 
 	/* copy the right parts to the right places */
-	STRNDUP(depend->name, depstring, ptr - depstring,
-			RET_ERR(PM_ERR_MEMORY, NULL));
+	if(ptr) {
+		STRNDUP(depend->name, depstring, ptr - depstring, return NULL);
+	} else {
+		STRDUP(depend->name, depstring, return NULL);
+	}
 	depend->name_hash = _alpm_hash_sdbm(depend->name);
 	if(version) {
-		STRDUP(depend->version, version, RET_ERR(PM_ERR_MEMORY, NULL));
+		STRDUP(depend->version, version, return NULL);
 	}
 
-	return(depend);
+	return depend;
 }
 
-pmdepend_t *_alpm_dep_dup(const pmdepend_t *dep)
+alpm_depend_t *_alpm_dep_dup(const alpm_depend_t *dep)
 {
-	pmdepend_t *newdep;
-	CALLOC(newdep, 1, sizeof(pmdepend_t), RET_ERR(PM_ERR_MEMORY, NULL));
+	alpm_depend_t *newdep;
+	CALLOC(newdep, 1, sizeof(alpm_depend_t), return NULL);
 
-	STRDUP(newdep->name, dep->name, RET_ERR(PM_ERR_MEMORY, NULL));
+	STRDUP(newdep->name, dep->name, return NULL);
 	newdep->name_hash = dep->name_hash;
-	STRDUP(newdep->version, dep->version, RET_ERR(PM_ERR_MEMORY, NULL));
+	STRDUP(newdep->version, dep->version, return NULL);
 	newdep->mod = dep->mod;
 
-	return(newdep);
+	return newdep;
 }
 
 /* These parameters are messy. We check if this package, given a list of
  * targets and a db is safe to remove. We do NOT remove it if it is in the
  * target list, or if if the package was explictly installed and
  * include_explicit == 0 */
-static int can_remove_package(pmdb_t *db, pmpkg_t *pkg, alpm_list_t *targets,
-		int include_explicit)
+static int can_remove_package(alpm_db_t *db, alpm_pkg_t *pkg,
+		alpm_list_t *targets, int include_explicit)
 {
 	alpm_list_t *i;
 
-	if(_alpm_pkg_find(targets, alpm_pkg_get_name(pkg))) {
-		return(0);
+	if(_alpm_pkg_find(targets, pkg->name)) {
+		return 0;
 	}
 
 	if(!include_explicit) {
 		/* see if it was explicitly installed */
-		if(alpm_pkg_get_reason(pkg) == PM_PKG_REASON_EXPLICIT) {
-			_alpm_log(PM_LOG_DEBUG, "excluding %s -- explicitly installed\n",
-					alpm_pkg_get_name(pkg));
-			return(0);
+		if(alpm_pkg_get_reason(pkg) == ALPM_PKG_REASON_EXPLICIT) {
+			_alpm_log(db->handle, ALPM_LOG_DEBUG,
+					"excluding %s -- explicitly installed\n", pkg->name);
+			return 0;
 		}
 	}
 
@@ -497,14 +505,14 @@ static int can_remove_package(pmdb_t *db, pmpkg_t *pkg, alpm_list_t *targets,
 
 	/* see if other packages need it */
 	for(i = _alpm_db_get_pkgcache(db); i; i = i->next) {
-		pmpkg_t *lpkg = i->data;
+		alpm_pkg_t *lpkg = i->data;
 		if(_alpm_dep_edge(lpkg, pkg) && !_alpm_pkg_find(targets, lpkg->name)) {
-			return(0);
+			return 0;
 		}
 	}
 
 	/* it's ok to remove */
-	return(1);
+	return 1;
 }
 
 /**
@@ -516,35 +524,40 @@ static int can_remove_package(pmdb_t *db, pmpkg_t *pkg, alpm_list_t *targets,
  * @param db package database to do dependency tracing in
  * @param *targs pointer to a list of packages
  * @param include_explicit if 0, explicitly installed packages are not included
+ * @return 0 on success, -1 on errors
  */
-void _alpm_recursedeps(pmdb_t *db, alpm_list_t *targs, int include_explicit)
+int _alpm_recursedeps(alpm_db_t *db, alpm_list_t *targs, int include_explicit)
 {
 	alpm_list_t *i, *j;
 
-	ALPM_LOG_FUNC;
-
 	if(db == NULL || targs == NULL) {
-		return;
+		return -1;
 	}
 
 	for(i = targs; i; i = i->next) {
-		pmpkg_t *pkg = i->data;
+		alpm_pkg_t *pkg = i->data;
 		for(j = _alpm_db_get_pkgcache(db); j; j = j->next) {
-			pmpkg_t *deppkg = j->data;
+			alpm_pkg_t *deppkg = j->data;
 			if(_alpm_dep_edge(pkg, deppkg)
 					&& can_remove_package(db, deppkg, targs, include_explicit)) {
-				_alpm_log(PM_LOG_DEBUG, "adding '%s' to the targets\n",
-						alpm_pkg_get_name(deppkg));
+				alpm_pkg_t *copy;
+				_alpm_log(db->handle, ALPM_LOG_DEBUG, "adding '%s' to the targets\n",
+						deppkg->name);
 				/* add it to the target list */
-				targs = alpm_list_add(targs, _alpm_pkg_dup(deppkg));
+				if(_alpm_pkg_dup(deppkg, &copy)) {
+					return -1;
+				}
+				targs = alpm_list_add(targs, copy);
 			}
 		}
 	}
+	return 0;
 }
 
 /**
  * helper function for resolvedeps: search for dep satisfier in dbs
  *
+ * @param handle the context handle
  * @param dep is the dependency to search for
  * @param dbs are the databases to search
  * @param excluding are the packages to exclude from the search
@@ -554,8 +567,8 @@ void _alpm_recursedeps(pmdb_t *db, alpm_list_t *targs, int include_explicit)
  *        an error code without prompting
  * @return the resolved package
  **/
-static pmpkg_t *resolvedep(pmdepend_t *dep, alpm_list_t *dbs,
-		alpm_list_t *excluding, int prompt)
+static alpm_pkg_t *resolvedep(alpm_handle_t *handle, alpm_depend_t *dep,
+		alpm_list_t *dbs, alpm_list_t *excluding, int prompt)
 {
 	alpm_list_t *i, *j;
 	int ignored = 0;
@@ -565,44 +578,49 @@ static pmpkg_t *resolvedep(pmdepend_t *dep, alpm_list_t *dbs,
 
 	/* 1. literals */
 	for(i = dbs; i; i = i->next) {
-		pmpkg_t *pkg = _alpm_db_get_pkgfromcache(i->data, dep->name);
-		if(pkg && _alpm_depcmp(pkg, dep) && !_alpm_pkg_find(excluding, pkg->name)) {
-			if(_alpm_pkg_should_ignore(pkg)) {
+		alpm_pkg_t *pkg = _alpm_db_get_pkgfromcache(i->data, dep->name);
+		if(pkg && _alpm_depcmp_literal(pkg, dep)
+				&& !_alpm_pkg_find(excluding, pkg->name)) {
+			if(_alpm_pkg_should_ignore(handle, pkg)) {
 				int install = 0;
-				if (prompt) {
-					QUESTION(handle->trans, PM_TRANS_CONV_INSTALL_IGNOREPKG, pkg,
+				if(prompt) {
+					QUESTION(handle, ALPM_QUESTION_INSTALL_IGNOREPKG, pkg,
 							 NULL, NULL, &install);
 				} else {
-					_alpm_log(PM_LOG_WARNING, _("ignoring package %s-%s\n"), pkg->name, pkg->version);
+					_alpm_log(handle, ALPM_LOG_WARNING, _("ignoring package %s-%s\n"),
+							pkg->name, pkg->version);
 				}
 				if(!install) {
 					ignored = 1;
 					continue;
 				}
 			}
-			return(pkg);
+			return pkg;
 		}
 	}
 	/* 2. satisfiers (skip literals here) */
 	for(i = dbs; i; i = i->next) {
 		for(j = _alpm_db_get_pkgcache(i->data); j; j = j->next) {
-			pmpkg_t *pkg = j->data;
-			if(_alpm_depcmp(pkg, dep) && strcmp(pkg->name, dep->name) != 0 &&
-			             !_alpm_pkg_find(excluding, pkg->name)) {
-				if(_alpm_pkg_should_ignore(pkg)) {
+			alpm_pkg_t *pkg = j->data;
+			/* with hash != hash, we can even skip the strcmp() as we know they can't
+			 * possibly be the same string */
+			if(pkg->name_hash != dep->name_hash && _alpm_depcmp(pkg, dep)
+					&& !_alpm_pkg_find(excluding, pkg->name)) {
+				if(_alpm_pkg_should_ignore(handle, pkg)) {
 					int install = 0;
-					if (prompt) {
-						QUESTION(handle->trans, PM_TRANS_CONV_INSTALL_IGNOREPKG,
+					if(prompt) {
+						QUESTION(handle, ALPM_QUESTION_INSTALL_IGNOREPKG,
 									pkg, NULL, NULL, &install);
 					} else {
-						_alpm_log(PM_LOG_WARNING, _("ignoring package %s-%s\n"), pkg->name, pkg->version);
+						_alpm_log(handle, ALPM_LOG_WARNING, _("ignoring package %s-%s\n"),
+								pkg->name, pkg->version);
 					}
 					if(!install) {
 						ignored = 1;
 						continue;
 					}
 				}
-				_alpm_log(PM_LOG_DEBUG, "provider found (%s provides %s)\n",
+				_alpm_log(handle, ALPM_LOG_DEBUG, "provider found (%s provides %s)\n",
 						pkg->name, dep->name);
 				providers = alpm_list_add(providers, pkg);
 				/* keep looking for other providers in the all dbs */
@@ -612,67 +630,72 @@ static pmpkg_t *resolvedep(pmdepend_t *dep, alpm_list_t *dbs,
 
 	/* first check if one provider is already installed locally */
 	for(i = providers; i; i = i->next) {
-		pmpkg_t *pkg = i->data;
-		if (_alpm_pkghash_find(_alpm_db_get_pkgcache_hash(handle->db_local), pkg->name)) {
+		alpm_pkg_t *pkg = i->data;
+		if(_alpm_pkghash_find(_alpm_db_get_pkgcache_hash(handle->db_local), pkg->name)) {
 			alpm_list_free(providers);
-			return(pkg);
+			return pkg;
 		}
 	}
 	count = alpm_list_count(providers);
-	if (count >= 1) {
+	if(count >= 1) {
 		/* default to first provider if there is no QUESTION callback */
 		int index = 0;
 		if(count > 1) {
 			/* if there is more than one provider, we ask the user */
-			QUESTION(handle->trans, PM_TRANS_CONV_SELECT_PROVIDER,
+			QUESTION(handle, ALPM_QUESTION_SELECT_PROVIDER,
 					providers, dep, NULL, &index);
 		}
 		if(index >= 0 && index < count) {
-			pmpkg_t *pkg = alpm_list_getdata(alpm_list_nth(providers, index));
+			alpm_list_t *nth = alpm_list_nth(providers, index);
+			alpm_pkg_t *pkg = nth->data;
 			alpm_list_free(providers);
-			return(pkg);
+			return pkg;
 		}
 		alpm_list_free(providers);
 		providers = NULL;
 	}
 
 	if(ignored) { /* resolvedeps will override these */
-		pm_errno = PM_ERR_PKG_IGNORED;
+		handle->pm_errno = ALPM_ERR_PKG_IGNORED;
 	} else {
-		pm_errno = PM_ERR_PKG_NOT_FOUND;
+		handle->pm_errno = ALPM_ERR_PKG_NOT_FOUND;
 	}
-	return(NULL);
+	return NULL;
 }
 
 /** Find a package satisfying a specified dependency.
  * First look for a literal, going through each db one by one. Then look for
  * providers. The first satisfier found is returned.
  * The dependency can include versions with depmod operators.
- * @param dbs an alpm_list_t* of pmdb_t where the satisfier will be searched
+ * @param handle the context handle
+ * @param dbs an alpm_list_t* of alpm_db_t where the satisfier will be searched
  * @param depstring package or provision name, versioned or not
- * @return a pmpkg_t* satisfying depstring
+ * @return a alpm_pkg_t* satisfying depstring
  */
-pmpkg_t SYMEXPORT *alpm_find_dbs_satisfier(alpm_list_t *dbs, const char *depstring)
+alpm_pkg_t SYMEXPORT *alpm_find_dbs_satisfier(alpm_handle_t *handle,
+		alpm_list_t *dbs, const char *depstring)
 {
-	pmdepend_t *dep;
-	pmpkg_t *pkg;
+	alpm_depend_t *dep;
+	alpm_pkg_t *pkg;
 
-	ASSERT(dbs, return(NULL));
+	CHECK_HANDLE(handle, return NULL);
+	ASSERT(dbs, RET_ERR(handle, ALPM_ERR_WRONG_ARGS, NULL));
 
 	dep = _alpm_splitdep(depstring);
-	ASSERT(dep, return(NULL));
-	pkg = resolvedep(dep, dbs, NULL, 1);
+	ASSERT(dep, return NULL);
+	pkg = resolvedep(handle, dep, dbs, NULL, 1);
 	_alpm_dep_free(dep);
-	return(pkg);
+	return pkg;
 }
 
 /**
  * Computes resolvable dependencies for a given package and adds that package
  * and those resolvable dependencies to a list.
  *
+ * @param handle the context handle
  * @param localpkgs is the list of local packages
- * @param dbs_sync are the sync databases
  * @param pkg is the package to resolve
+ * @param preferred packages to prefer when resolving
  * @param packages is a pointer to a list of packages which will be
  *        searched first for any dependency packages needed to complete the
  *        resolve, and to which will be added any [pkg] and all of its
@@ -686,9 +709,9 @@ pmpkg_t SYMEXPORT *alpm_find_dbs_satisfier(alpm_list_t *dbs, const char *depstri
  *         unresolvable dependency, in which case the [*packages] list will be
  *         unmodified by this function
  */
-int _alpm_resolvedeps(alpm_list_t *localpkgs, alpm_list_t *dbs_sync, pmpkg_t *pkg,
-                      alpm_list_t *preferred, alpm_list_t **packages,
-                      alpm_list_t *remove, alpm_list_t **data)
+int _alpm_resolvedeps(alpm_handle_t *handle, alpm_list_t *localpkgs,
+		alpm_pkg_t *pkg, alpm_list_t *preferred, alpm_list_t **packages,
+		alpm_list_t *remove, alpm_list_t **data)
 {
 	int ret = 0;
 	alpm_list_t *i, *j;
@@ -696,10 +719,14 @@ int _alpm_resolvedeps(alpm_list_t *localpkgs, alpm_list_t *dbs_sync, pmpkg_t *pk
 	alpm_list_t *deps = NULL;
 	alpm_list_t *packages_copy;
 
-	ALPM_LOG_FUNC;
-
 	if(_alpm_pkg_find(*packages, pkg->name) != NULL) {
-		return(0);
+		return 0;
+	}
+
+	if(handle->trans->flags & ALPM_TRANS_FLAG_RECURSE) {
+		/* removing local packages from the equation causes the entire dep chain to
+		 * get pulled for each target- e.g., pactree -u output */
+		localpkgs = NULL;
 	}
 
 	/* Create a copy of the packages list, so that it can be restored
@@ -709,16 +736,16 @@ int _alpm_resolvedeps(alpm_list_t *localpkgs, alpm_list_t *dbs_sync, pmpkg_t *pk
 	   on that list */
 	*packages = alpm_list_add(*packages, pkg);
 
-	_alpm_log(PM_LOG_DEBUG, "started resolving dependencies\n");
+	_alpm_log(handle, ALPM_LOG_DEBUG, "started resolving dependencies\n");
 	for(i = alpm_list_last(*packages); i; i = i->next) {
-		pmpkg_t *tpkg = i->data;
+		alpm_pkg_t *tpkg = i->data;
 		targ = alpm_list_add(NULL, tpkg);
-		deps = alpm_checkdeps(localpkgs, 0, remove, targ);
+		deps = alpm_checkdeps(handle, localpkgs, remove, targ, 0);
 		alpm_list_free(targ);
 
 		for(j = deps; j; j = j->next) {
-			pmdepmissing_t *miss = j->data;
-			pmdepend_t *missdep = alpm_miss_get_dep(miss);
+			alpm_depmissing_t *miss = j->data;
+			alpm_depend_t *missdep = miss->depend;
 			/* check if one of the packages in the [*packages] list already satisfies
 			 * this dependency */
 			if(find_dep_satisfier(*packages, missdep)) {
@@ -727,15 +754,15 @@ int _alpm_resolvedeps(alpm_list_t *localpkgs, alpm_list_t *dbs_sync, pmpkg_t *pk
 			}
 			/* check if one of the packages in the [preferred] list already satisfies
 			 * this dependency */
-			pmpkg_t *spkg = find_dep_satisfier(preferred, missdep);
+			alpm_pkg_t *spkg = find_dep_satisfier(preferred, missdep);
 			if(!spkg) {
 				/* find a satisfier package in the given repositories */
-				spkg = resolvedep(missdep, dbs_sync, *packages, 0);
+				spkg = resolvedep(handle, missdep, handle->dbs_sync, *packages, 0);
 			}
 			if(!spkg) {
-				pm_errno = PM_ERR_UNSATISFIED_DEPS;
+				handle->pm_errno = ALPM_ERR_UNSATISFIED_DEPS;
 				char *missdepstring = alpm_dep_compute_string(missdep);
-				_alpm_log(PM_LOG_WARNING,
+				_alpm_log(handle, ALPM_LOG_WARNING,
 						_("cannot resolve \"%s\", a dependency of \"%s\"\n"),
 						missdepstring, tpkg->name);
 				free(missdepstring);
@@ -744,8 +771,9 @@ int _alpm_resolvedeps(alpm_list_t *localpkgs, alpm_list_t *dbs_sync, pmpkg_t *pk
 				}
 				ret = -1;
 			} else {
-				_alpm_log(PM_LOG_DEBUG, "pulling dependency %s (needed by %s)\n",
-						alpm_pkg_get_name(spkg), alpm_pkg_get_name(tpkg));
+				_alpm_log(handle, ALPM_LOG_DEBUG,
+						"pulling dependency %s (needed by %s)\n",
+						spkg->name, tpkg->name);
 				*packages = alpm_list_add(*packages, spkg);
 				_alpm_depmiss_free(miss);
 			}
@@ -753,91 +781,51 @@ int _alpm_resolvedeps(alpm_list_t *localpkgs, alpm_list_t *dbs_sync, pmpkg_t *pk
 		alpm_list_free(deps);
 	}
 
+	if(handle->trans->flags & ALPM_TRANS_FLAG_NEEDED) {
+		/* remove any deps that were pulled that match installed version */
+		/* odd loop syntax so we can modify the list as we iterate */
+		i = *packages;
+		while(i) {
+			alpm_pkg_t *tpkg = i->data;
+			alpm_pkg_t *local = _alpm_db_get_pkgfromcache(
+					handle->db_local, tpkg->name);
+			if(local && _alpm_pkg_compare_versions(tpkg, local) == 0) {
+				/* with the NEEDED flag, packages up to date are not reinstalled */
+				_alpm_log(handle, ALPM_LOG_DEBUG,
+						"not adding dep %s-%s as it is not needed, same version\n",
+						local->name, local->version);
+				j = i;
+				i = i->next;
+				*packages = alpm_list_remove_item(*packages, j);
+				free(j);
+			} else {
+				i = i->next;
+			}
+		}
+	}
+
 	if(ret != 0) {
 		alpm_list_free(*packages);
 		*packages = packages_copy;
 	} else {
 		alpm_list_free(packages_copy);
 	}
-	_alpm_log(PM_LOG_DEBUG, "finished resolving dependencies\n");
-	return(ret);
+	_alpm_log(handle, ALPM_LOG_DEBUG, "finished resolving dependencies\n");
+	return ret;
 }
 
-const char SYMEXPORT *alpm_miss_get_target(const pmdepmissing_t *miss)
-{
-	ALPM_LOG_FUNC;
-
-	/* Sanity checks */
-	ASSERT(miss != NULL, return(NULL));
-
-	return(miss->target);
-}
-
-const char SYMEXPORT *alpm_miss_get_causingpkg(const pmdepmissing_t *miss)
-{
-	ALPM_LOG_FUNC;
-
-	/* Sanity checks */
-	ASSERT(miss != NULL, return(NULL));
-
-	return miss->causingpkg;
-}
-
-pmdepend_t SYMEXPORT *alpm_miss_get_dep(pmdepmissing_t *miss)
-{
-	ALPM_LOG_FUNC;
-
-	/* Sanity checks */
-	ASSERT(miss != NULL, return(NULL));
-
-	return(miss->depend);
-}
-
-pmdepmod_t SYMEXPORT alpm_dep_get_mod(const pmdepend_t *dep)
-{
-	ALPM_LOG_FUNC;
-
-	/* Sanity checks */
-	ASSERT(dep != NULL, return(-1));
-
-	return(dep->mod);
-}
-
-const char SYMEXPORT *alpm_dep_get_name(const pmdepend_t *dep)
-{
-	ALPM_LOG_FUNC;
-
-	/* Sanity checks */
-	ASSERT(dep != NULL, return(NULL));
-
-	return(dep->name);
-}
-
-const char SYMEXPORT *alpm_dep_get_version(const pmdepend_t *dep)
-{
-	ALPM_LOG_FUNC;
-
-	/* Sanity checks */
-	ASSERT(dep != NULL, return(NULL));
-
-	return(dep->version);
-}
-
-/** Reverse of splitdep; make a dep string from a pmdepend_t struct.
+/** Reverse of splitdep; make a dep string from a alpm_depend_t struct.
  * The string must be freed!
  * @param dep the depend to turn into a string
  * @return a string-formatted dependency with operator if necessary
  */
-char SYMEXPORT *alpm_dep_compute_string(const pmdepend_t *dep)
+char SYMEXPORT *alpm_dep_compute_string(const alpm_depend_t *dep)
 {
 	const char *name, *opr, *ver;
 	char *str;
 	size_t len;
 
-	ALPM_LOG_FUNC;
-
-	/* Sanity checks */
-	ASSERT(dep != NULL, return(NULL));
+	ASSERT(dep != NULL, return NULL);
 
 	if(dep->name) {
 		name = dep->name;
@@ -846,22 +834,22 @@ char SYMEXPORT *alpm_dep_compute_string(const pmdepend_t *dep)
 	}
 
 	switch(dep->mod) {
-		case PM_DEP_MOD_ANY:
+		case ALPM_DEP_MOD_ANY:
 			opr = "";
 			break;
-		case PM_DEP_MOD_GE:
+		case ALPM_DEP_MOD_GE:
 			opr = ">=";
 			break;
-		case PM_DEP_MOD_LE:
+		case ALPM_DEP_MOD_LE:
 			opr = "<=";
 			break;
-		case PM_DEP_MOD_EQ:
+		case ALPM_DEP_MOD_EQ:
 			opr = "=";
 			break;
-		case PM_DEP_MOD_LT:
+		case ALPM_DEP_MOD_LT:
 			opr = "<";
 			break;
-		case PM_DEP_MOD_GT:
+		case ALPM_DEP_MOD_GT:
 			opr = ">";
 			break;
 		default:
@@ -869,19 +857,19 @@ char SYMEXPORT *alpm_dep_compute_string(const pmdepend_t *dep)
 			break;
 	}
 
-	if(dep->mod != PM_DEP_MOD_ANY && dep->version) {
+	if(dep->mod != ALPM_DEP_MOD_ANY && dep->version) {
 		ver = dep->version;
 	} else {
 		ver = "";
 	}
 
 	/* we can always compute len and print the string like this because opr
-	 * and ver will be empty when PM_DEP_MOD_ANY is the depend type. the
+	 * and ver will be empty when ALPM_DEP_MOD_ANY is the depend type. the
 	 * reassignments above also ensure we do not do a strlen(NULL). */
 	len = strlen(name) + strlen(opr) + strlen(ver) + 1;
-	MALLOC(str, len, RET_ERR(PM_ERR_MEMORY, NULL));
+	MALLOC(str, len, return NULL);
 	snprintf(str, len, "%s%s%s", name, opr, ver);
 
-	return(str);
+	return str;
 }
 /* vim: set ts=2 sw=2 noet: */
