@@ -25,14 +25,28 @@
 
 #include <time.h>
 
-#define PM_DLBUF_LEN (1024 * 16)
+struct dload_payload {
+	alpm_handle_t *handle;
+	const char *tempfile_openmode;
+	char *remote_name;
+	char *tempfile_name;
+	char *destfile_name;
+	char *content_disp_name;
+	char *fileurl;
+	off_t initial_size;
+	off_t max_size;
+	off_t prevprogress;
+	int force;
+	int allow_resume;
+	int errors_ok;
+	int unlink_on_fail;
+	CURLcode curlerr;       /* last error produced by curl */
+};
 
-int _alpm_download_single_file(const char *filename,
-		alpm_list_t *servers, const char *localpath,
-		int force);
+void _alpm_dload_payload_reset(struct dload_payload *payload);
 
-int _alpm_download_files(alpm_list_t *files,
-		alpm_list_t *servers, const char *localpath);
+int _alpm_download(struct dload_payload *payload, const char *localpath,
+		char **final_file);
 
 #endif /* _ALPM_DLOAD_H */
 
