@@ -37,14 +37,8 @@
 #include "conf.h"
 
 /* download progress bar */
-static double rate_last;
-static off_t xfered_last;
 static off_t list_xfered = 0.0;
 static off_t list_total = 0.0;
-static struct timeval initial_time;
-
-/* transaction progress bar */
-static int prevpercent = 0; /* for less progressbar output */
 
 /* delayed output during progress bar */
 static int on_progress = 0;
@@ -82,7 +76,7 @@ static double get_update_timediff(int first_call)
 		}
 	}
 
-	return(retval);
+	return retval;
 }
 
 /* refactored from cb_trans_progress */
@@ -154,96 +148,105 @@ static void fill_progress(const int bar_percent, const int disp_percent,
 
 
 /* callback to handle messages/notifications from libalpm transactions */
-void cb_trans_evt(pmtransevt_t event, void *data1, void *data2)
+void cb_event(alpm_event_t event, void *data1, void *data2)
 {
+	if(config->print) {
+		return;
+	}
 	switch(event) {
-		case PM_TRANS_EVT_CHECKDEPS_START:
+		case ALPM_EVENT_CHECKDEPS_START:
 		  printf(_("checking dependencies...\n"));
 			break;
-		case PM_TRANS_EVT_FILECONFLICTS_START:
+		case ALPM_EVENT_FILECONFLICTS_START:
 			if(config->noprogressbar) {
 				printf(_("checking for file conflicts...\n"));
 			}
 			break;
-		case PM_TRANS_EVT_RESOLVEDEPS_START:
+		case ALPM_EVENT_RESOLVEDEPS_START:
 			printf(_("resolving dependencies...\n"));
 			break;
-		case PM_TRANS_EVT_INTERCONFLICTS_START:
+		case ALPM_EVENT_INTERCONFLICTS_START:
 			printf(_("looking for inter-conflicts...\n"));
 			break;
-		case PM_TRANS_EVT_ADD_START:
+		case ALPM_EVENT_ADD_START:
 			if(config->noprogressbar) {
 				printf(_("installing %s...\n"), alpm_pkg_get_name(data1));
 			}
 			break;
-		case PM_TRANS_EVT_ADD_DONE:
-			alpm_logaction("installed %s (%s)\n",
+		case ALPM_EVENT_ADD_DONE:
+			alpm_logaction(config->handle, "installed %s (%s)\n",
 			         alpm_pkg_get_name(data1),
 			         alpm_pkg_get_version(data1));
 			display_optdepends(data1);
 			break;
-		case PM_TRANS_EVT_REMOVE_START:
+		case ALPM_EVENT_REMOVE_START:
 			if(config->noprogressbar) {
 			printf(_("removing %s...\n"), alpm_pkg_get_name(data1));
 			}
 			break;
-		case PM_TRANS_EVT_REMOVE_DONE:
-			alpm_logaction("removed %s (%s)\n",
+		case ALPM_EVENT_REMOVE_DONE:
+			alpm_logaction(config->handle, "removed %s (%s)\n",
 			         alpm_pkg_get_name(data1),
 			         alpm_pkg_get_version(data1));
 			break;
-		case PM_TRANS_EVT_UPGRADE_START:
+		case ALPM_EVENT_UPGRADE_START:
 			if(config->noprogressbar) {
 				printf(_("upgrading %s...\n"), alpm_pkg_get_name(data1));
 			}
 			break;
-		case PM_TRANS_EVT_UPGRADE_DONE:
-			alpm_logaction("upgraded %s (%s -> %s)\n",
+		case ALPM_EVENT_UPGRADE_DONE:
+			alpm_logaction(config->handle, "upgraded %s (%s -> %s)\n",
 			         (char *)alpm_pkg_get_name(data1),
 			         (char *)alpm_pkg_get_version(data2),
 			         (char *)alpm_pkg_get_version(data1));
 			display_new_optdepends(data2,data1);
 			break;
-		case PM_TRANS_EVT_INTEGRITY_START:
+		case ALPM_EVENT_INTEGRITY_START:
 			if(config->noprogressbar) {
 				printf(_("checking package integrity...\n"));
 			}
 			break;
-		case PM_TRANS_EVT_DELTA_INTEGRITY_START:
+		case ALPM_EVENT_LOAD_START:
+			if(config->noprogressbar) {
+				printf(_("loading package files...\n"));
+			}
+			break;
+		case ALPM_EVENT_DELTA_INTEGRITY_START:
 			printf(_("checking delta integrity...\n"));
 			break;
-		case PM_TRANS_EVT_DELTA_PATCHES_START:
+		case ALPM_EVENT_DELTA_PATCHES_START:
 			printf(_("applying deltas...\n"));
 			break;
-		case PM_TRANS_EVT_DELTA_PATCH_START:
+		case ALPM_EVENT_DELTA_PATCH_START:
 			printf(_("generating %s with %s... "), (char *)data1, (char *)data2);
 			break;
-		case PM_TRANS_EVT_DELTA_PATCH_DONE:
+		case ALPM_EVENT_DELTA_PATCH_DONE:
 			printf(_("success!\n"));
 			break;
-		case PM_TRANS_EVT_DELTA_PATCH_FAILED:
+		case ALPM_EVENT_DELTA_PATCH_FAILED:
 			printf(_("failed.\n"));
 			break;
-		case PM_TRANS_EVT_SCRIPTLET_INFO:
-			printf("%s", (char*)data1);
+		case ALPM_EVENT_SCRIPTLET_INFO:
+			printf("%s", (char *)data1);
 			break;
-		case PM_TRANS_EVT_RETRIEVE_START:
-			printf(_(":: Retrieving packages from %s...\n"), (char*)data1);
+		case ALPM_EVENT_RETRIEVE_START:
+			printf(_(":: Retrieving packages from %s...\n"), (char *)data1);
 			break;
-		case PM_TRANS_EVT_DISKSPACE_START:
+		case ALPM_EVENT_DISKSPACE_START:
 			if(config->noprogressbar) {
 				printf(_("checking available disk space...\n"));
 			}
 			break;
 		/* all the simple done events, with fallthrough for each */
-		case PM_TRANS_EVT_FILECONFLICTS_DONE:
-		case PM_TRANS_EVT_CHECKDEPS_DONE:
-		case PM_TRANS_EVT_RESOLVEDEPS_DONE:
-		case PM_TRANS_EVT_INTERCONFLICTS_DONE:
-		case PM_TRANS_EVT_INTEGRITY_DONE:
-		case PM_TRANS_EVT_DELTA_INTEGRITY_DONE:
-		case PM_TRANS_EVT_DELTA_PATCHES_DONE:
-		case PM_TRANS_EVT_DISKSPACE_DONE:
+		case ALPM_EVENT_FILECONFLICTS_DONE:
+		case ALPM_EVENT_CHECKDEPS_DONE:
+		case ALPM_EVENT_RESOLVEDEPS_DONE:
+		case ALPM_EVENT_INTERCONFLICTS_DONE:
+		case ALPM_EVENT_INTEGRITY_DONE:
+		case ALPM_EVENT_LOAD_DONE:
+		case ALPM_EVENT_DELTA_INTEGRITY_DONE:
+		case ALPM_EVENT_DELTA_PATCHES_DONE:
+		case ALPM_EVENT_DISKSPACE_DONE:
 			/* nothing */
 			break;
 	}
@@ -252,21 +255,28 @@ void cb_trans_evt(pmtransevt_t event, void *data1, void *data2)
 
 /* callback to handle questions from libalpm transactions (yes/no) */
 /* TODO this is one of the worst ever functions written. void *data ? wtf */
-void cb_trans_conv(pmtransconv_t event, void *data1, void *data2,
+void cb_question(alpm_question_t event, void *data1, void *data2,
                    void *data3, int *response)
 {
+	if(config->print) {
+		return;
+	}
 	switch(event) {
-		case PM_TRANS_CONV_INSTALL_IGNOREPKG:
-			*response = yesno(_(":: %s is in IgnorePkg/IgnoreGroup. Install anyway?"),
-							  alpm_pkg_get_name(data1));
+		case ALPM_QUESTION_INSTALL_IGNOREPKG:
+			if(!config->op_s_downloadonly) {
+				*response = yesno(_(":: %s is in IgnorePkg/IgnoreGroup. Install anyway?"),
+								  alpm_pkg_get_name(data1));
+			} else {
+				*response = 1;
+			}
 			break;
-		case PM_TRANS_CONV_REPLACE_PKG:
+		case ALPM_QUESTION_REPLACE_PKG:
 			*response = yesno(_(":: Replace %s with %s/%s?"),
 					alpm_pkg_get_name(data1),
 					(char *)data3,
 					alpm_pkg_get_name(data2));
 			break;
-		case PM_TRANS_CONV_CONFLICT_PKG:
+		case ALPM_QUESTION_CONFLICT_PKG:
 			/* data parameters: target package, local package, conflict (strings) */
 			/* print conflict only if it contains new information */
 			if(strcmp(data1, data3) == 0 || strcmp(data2, data3) == 0) {
@@ -282,7 +292,7 @@ void cb_trans_conv(pmtransconv_t event, void *data1, void *data2,
 						(char *)data2);
 			}
 			break;
-		case PM_TRANS_CONV_REMOVE_PKGS:
+		case ALPM_QUESTION_REMOVE_PKGS:
 			{
 				alpm_list_t *unresolved = (alpm_list_t *) data1;
 				alpm_list_t *namelist = NULL, *i;
@@ -305,19 +315,19 @@ void cb_trans_conv(pmtransconv_t event, void *data1, void *data2,
 				alpm_list_free(namelist);
 			}
 			break;
-		case PM_TRANS_CONV_SELECT_PROVIDER:
+		case ALPM_QUESTION_SELECT_PROVIDER:
 			{
 				alpm_list_t *providers = (alpm_list_t *)data1;
-				int count = alpm_list_count(providers);
-				char *depstring = alpm_dep_compute_string((pmdepend_t *)data2);
-				printf(_(":: There are %d providers available for %s:\n"), count,
+				size_t count = alpm_list_count(providers);
+				char *depstring = alpm_dep_compute_string((alpm_depend_t *)data2);
+				printf(_(":: There are %zd providers available for %s:\n"), count,
 						depstring);
 				free(depstring);
 				select_display(providers);
 				*response = select_question(count);
 			}
 			break;
-		case PM_TRANS_CONV_LOCAL_NEWER:
+		case ALPM_QUESTION_LOCAL_NEWER:
 			if(!config->op_s_downloadonly) {
 				*response = yesno(_(":: %s-%s: local version is newer. Upgrade anyway?"),
 						alpm_pkg_get_name(data1),
@@ -326,9 +336,20 @@ void cb_trans_conv(pmtransconv_t event, void *data1, void *data2,
 				*response = 1;
 			}
 			break;
-		case PM_TRANS_CONV_CORRUPTED_PKG:
-			*response = yesno(_(":: File %s is corrupted. Do you want to delete it?"),
-					(char *)data1);
+		case ALPM_QUESTION_CORRUPTED_PKG:
+			*response = yesno(_(":: File %s is corrupted (%s).\n"
+						"Do you want to delete it?"),
+					(char *)data1,
+					alpm_strerror(*(enum _alpm_errno_t *)data2));
+			break;
+		case ALPM_QUESTION_IMPORT_KEY:
+			{
+				alpm_pgpkey_t *key = data1;
+				char created[12];
+				strftime(created, 12, "%Y-%m-%d", localtime(&(key->created)));
+				*response = yesno(_(":: Import PGP key %s, \"%s\", created %s?"),
+						key->fingerprint, key->uid, created);
+			}
 			break;
 	}
 	if(config->noask) {
@@ -340,9 +361,11 @@ void cb_trans_conv(pmtransconv_t event, void *data1, void *data2,
 }
 
 /* callback to handle display of transaction progress */
-void cb_trans_progress(pmtransprog_t event, const char *pkgname, int percent,
+void cb_progress(alpm_progress_t event, const char *pkgname, int percent,
                        size_t howmany, size_t current)
 {
+	static int prevpercent;
+	static size_t prevcurrent;
 	/* size of line to allocate for text printing (e.g. not progressbar) */
 	int infolen;
 	int digits, textlen;
@@ -352,7 +375,7 @@ void cb_trans_progress(pmtransprog_t event, const char *pkgname, int percent,
 	int len, wclen, wcwid, padwid;
 	wchar_t *wcstr;
 
-	const int cols = getcols(0);
+	const unsigned short cols = getcols();
 
 	if(config->noprogressbar || cols == 0) {
 		return;
@@ -367,41 +390,48 @@ void cb_trans_progress(pmtransprog_t event, const char *pkgname, int percent,
 			return;
 		}
 	} else {
-		if(!pkgname || percent == prevpercent || get_update_timediff(0) < UPDATE_SPEED_SEC) {
+		if(current != prevcurrent) {
+			/* update always */
+		} else if(!pkgname || percent == prevpercent ||
+				get_update_timediff(0) < UPDATE_SPEED_SEC) {
 			/* only update the progress bar when we have a package name, the
-			 * percentage has changed, and it has been long enough.  */
+			 * percentage has changed, and it has been long enough. */
 			return;
 		}
 	}
 
 	prevpercent = percent;
+	prevcurrent = current;
 
 	/* set text of message to display */
 	switch (event) {
-		case PM_TRANS_PROGRESS_ADD_START:
+		case ALPM_PROGRESS_ADD_START:
 			opr = _("installing");
 			break;
-		case PM_TRANS_PROGRESS_UPGRADE_START:
+		case ALPM_PROGRESS_UPGRADE_START:
 			opr = _("upgrading");
 			break;
-		case PM_TRANS_PROGRESS_REMOVE_START:
+		case ALPM_PROGRESS_REMOVE_START:
 			opr = _("removing");
 			break;
-		case PM_TRANS_PROGRESS_CONFLICTS_START:
+		case ALPM_PROGRESS_CONFLICTS_START:
 			opr = _("checking for file conflicts");
 			break;
-		case PM_TRANS_PROGRESS_DISKSPACE_START:
+		case ALPM_PROGRESS_DISKSPACE_START:
 			opr = _("checking available disk space");
 			break;
-		case PM_TRANS_PROGRESS_INTEGRITY_START:
+		case ALPM_PROGRESS_INTEGRITY_START:
 			opr = _("checking package integrity");
 			break;
+		case ALPM_PROGRESS_LOAD_START:
+			opr = _("loading package files");
+			break;
 		default:
 			return;
 	}
 
 	infolen = cols * 6 / 10;
-	if (infolen < 50) {
+	if(infolen < 50) {
 		infolen = 50;
 	}
 
@@ -486,6 +516,9 @@ void cb_dl_total(off_t total)
 /* callback to handle display of download progress */
 void cb_dl_progress(const char *filename, off_t file_xfered, off_t file_total)
 {
+	static double rate_last;
+	static off_t xfered_last;
+	static struct timeval initial_time;
 	int infolen;
 	int filenamelen;
 	char *fname, *p;
@@ -495,12 +528,13 @@ void cb_dl_progress(const char *filename, off_t file_xfered, off_t file_total)
 
 	int totaldownload = 0;
 	off_t xfered, total;
-	double rate = 0.0, timediff = 0.0, f_xfered = 0.0;
+	double rate = 0.0, timediff = 0.0;
 	unsigned int eta_h = 0, eta_m = 0, eta_s = 0;
+	double rate_human, xfered_human;
+	const char *rate_label, *xfered_label;
 	int file_percent = 0, total_percent = 0;
-	char rate_size = 'K', xfered_size = 'K';
 
-	const int cols = getcols(0);
+	const unsigned short cols = getcols();
 
 	if(config->noprogressbar || cols == 0 || file_total == -1) {
 		if(file_xfered == 0) {
@@ -511,12 +545,9 @@ void cb_dl_progress(const char *filename, off_t file_xfered, off_t file_total)
 	}
 
 	infolen = cols * 6 / 10;
-	if (infolen < 50) {
+	if(infolen < 50) {
 		infolen = 50;
 	}
-	/* explanation of magic 28 number at the end */
-	filenamelen = infolen - 28;
-
 	/* only use TotalDownload if enabled and we have a callback value */
 	if(config->totaldownload && list_total) {
 		/* sanity check */
@@ -563,7 +594,7 @@ void cb_dl_progress(const char *filename, off_t file_xfered, off_t file_total)
 		diff_usec = current_time.tv_usec - initial_time.tv_usec;
 		timediff = diff_sec + (diff_usec / 1000000.0);
 		if(timediff > 0.0) {
-			rate = xfered / (timediff * 1024.0);
+			rate = xfered / timediff;
 			/* round elapsed time to the nearest second */
 			eta_s = (unsigned int)(timediff + 0.5);
 		} else {
@@ -577,11 +608,11 @@ void cb_dl_progress(const char *filename, off_t file_xfered, off_t file_total)
 			/* return if the calling interval was too short */
 			return;
 		}
-		rate = (xfered - xfered_last) / (timediff * 1024.0);
+		rate = (xfered - xfered_last) / timediff;
 		/* average rate to reduce jumpiness */
 		rate = (rate + 2 * rate_last) / 3;
 		if(rate > 0.0) {
-			eta_s = (total - xfered) / (rate * 1024.0);
+			eta_s = (total - xfered) / rate;
 		} else {
 			eta_s = UINT_MAX;
 		}
@@ -611,69 +642,76 @@ void cb_dl_progress(const char *filename, off_t file_xfered, off_t file_total)
 	eta_m = eta_s / 60;
 	eta_s -= eta_m * 60;
 
-	fname = strdup(filename);
+	len = strlen(filename);
+	fname = malloc(len + 1);
+	memcpy(fname, filename, len);
 	/* strip package or DB extension for cleaner look */
 	if((p = strstr(fname, ".pkg")) || (p = strstr(fname, ".db"))) {
-			*p = '\0';
+		/* tack on a .sig suffix for signatures */
+		if(memcmp(&filename[len - 4], ".sig", 4) == 0) {
+			memcpy(p, ".sig", 4);
+
+			/* adjust length for later calculations */
+			len = p - fname + 4;
+		} else {
+			len = p - fname;
+		}
 	}
+	fname[len] = '\0';
+
+	/* 1 space + filenamelen + 1 space + 6 for size + 1 space + 3 for label +
+	 * + 2 spaces + 4 for rate  + 1 for label + 2 for /s + 1 space +
+	 * 8 for eta, gives us the magic 30 */
+	filenamelen = infolen - 30;
+	/* see printf() code, we omit 'HH:' in these conditions */
+	if(eta_h == 0 || eta_h >= 100) {
+		filenamelen += 3;
+	}
+
 	/* In order to deal with characters from all locales, we have to worry
 	 * about wide characters and their column widths. A lot of stuff is
 	 * done here to figure out the actual number of screen columns used
 	 * by the output, and then pad it accordingly so we fill the terminal.
 	 */
 	/* len = filename len + null */
-	len = strlen(filename) + 1;
-	wcfname = calloc(len, sizeof(wchar_t));
+	wcfname = calloc(len + 1, sizeof(wchar_t));
 	wclen = mbstowcs(wcfname, fname, len);
 	wcwid = wcswidth(wcfname, wclen);
 	padwid = filenamelen - wcwid;
 	/* if padwid is < 0, we need to trim the string so padwid = 0 */
 	if(padwid < 0) {
 		int i = filenamelen - 3;
-		wchar_t *p = wcfname;
+		wchar_t *wcp = wcfname;
 		/* grab the max number of char columns we can fill */
-		while(i > 0 && wcwidth(*p) < i) {
-			i -= wcwidth(*p);
-			p++;
+		while(i > 0 && wcwidth(*wcp) < i) {
+			i -= wcwidth(*wcp);
+			wcp++;
 		}
 		/* then add the ellipsis and fill out any extra padding */
-		wcscpy(p, L"...");
+		wcscpy(wcp, L"...");
 		padwid = i;
 
 	}
 
-	/* Awesome formatting for progress bar.  We need a mess of Kb->Mb->Gb stuff
-	 * here. We'll use limit of 2048 for each until we get some empirical */
-	/* rate_size = 'K'; was set above */
-	if(rate > 2048.0) {
-		rate /= 1024.0;
-		rate_size = 'M';
-		if(rate > 2048.0) {
-			rate /= 1024.0;
-			rate_size = 'G';
-			/* we should not go higher than this for a few years (9999.9 Gb/s?)*/
-		}
-	}
+	rate_human = humanize_size((off_t)rate, '\0', &rate_label);
+	xfered_human = humanize_size(xfered, '\0', &xfered_label);
 
-	f_xfered = xfered / 1024.0; /* convert to K by default */
-	/* xfered_size = 'K'; was set above */
-	if(f_xfered > 2048.0) {
-		f_xfered /= 1024.0;
-		xfered_size = 'M';
-		if(f_xfered > 2048.0) {
-			f_xfered /= 1024.0;
-			xfered_size = 'G';
-			/* I should seriously hope that archlinux packages never break
-			 * the 9999.9GB mark... we'd have more serious problems than the progress
-			 * bar in pacman */
-		}
+	printf(" %ls%-*s ", wcfname, padwid, "");
+	/* We will show 1.6M/s, 11.6M/s, but 116K/s and 1116K/s */
+	if(rate_human < 100.0) {
+		printf("%6.1f %3s  %4.1f%c/s ",
+				xfered_human, xfered_label, rate_human, rate_label[0]);
+	} else {
+		printf("%6.1f %3s  %4.f%c/s ",
+				xfered_human, xfered_label, rate_human, rate_label[0]);
+	}
+	if(eta_h == 0) {
+		printf("%02u:%02u", eta_m, eta_s);
+	} else if(eta_h < 100) {
+		printf("%02u:%02u:%02u", eta_h, eta_m, eta_s);
+	} else {
+		printf("--:--");
 	}
-
-	/* 1 space + filenamelen + 1 space + 7 for size + 1 + 7 for rate + 2 for /s + 1 space + 8 for eta */
-	/* TODO: if eta_h > 99, formatting gets all messed up */
-	printf(" %ls%-*s %6.1f%c %#6.1f%c/s %02u:%02u:%02u", wcfname,
-			padwid, "", f_xfered, xfered_size,
-			rate, rate_size, eta_h, eta_m, eta_s);
 
 	free(fname);
 	free(wcfname);
@@ -687,7 +725,7 @@ void cb_dl_progress(const char *filename, off_t file_xfered, off_t file_total)
 }
 
 /* Callback to handle notifications from the library */
-void cb_log(pmloglevel_t level, const char *fmt, va_list args)
+void cb_log(alpm_loglevel_t level, const char *fmt, va_list args)
 {
 	if(!fmt || strlen(fmt) == 0) {
 		return;
@@ -700,7 +738,7 @@ void cb_log(pmloglevel_t level, const char *fmt, va_list args)
 			output = alpm_list_add(output, string);
 		}
 	} else {
-		pm_vfprintf(stdout, level, fmt, args);
+		pm_vfprintf(stderr, level, fmt, args);
 	}
 }
 
