@@ -32,6 +32,10 @@ typedef struct __config_t {
 	unsigned short noprogressbar;
 	unsigned short logmask;
 	unsigned short print;
+	unsigned short checkspace;
+	unsigned short usesyslog;
+	unsigned short usedelta;
+	char *arch;
 	char *print_format;
 	/* unfortunately, we have to keep track of paths both here and in the library
 	 * because they can come from both the command line or config file, and we
@@ -40,7 +44,8 @@ typedef struct __config_t {
 	char *rootdir;
 	char *dbpath;
 	char *logfile;
-	/* TODO how to handle cachedirs? */
+	char *gpgdir;
+	alpm_list_t *cachedirs;
 
 	unsigned short op_q_isfile;
 	unsigned short op_q_info;
@@ -63,20 +68,34 @@ typedef struct __config_t {
 	unsigned short op_s_upgrade;
 
 	unsigned short group;
-	pmtransflag_t flags;
 	unsigned short noask;
 	unsigned int ask;
+	alpm_transflag_t flags;
+	alpm_siglevel_t siglevel;
 
 	/* conf file options */
-	unsigned short chomp; /* I Love Candy! */
-	unsigned short showsize; /* show individual package sizes */
+	/* I Love Candy! */
+	unsigned short chomp;
+	/* format target pkg lists as table */
+	unsigned short verbosepkglists;
 	/* When downloading, display the amount downloaded, rate, ETA, and percent
 	 * downloaded of the total download list */
 	unsigned short totaldownload;
-	unsigned short cleanmethod; /* select -Sc behavior */
+	/* select -Sc behavior */
+	unsigned short cleanmethod;
 	alpm_list_t *holdpkg;
 	alpm_list_t *syncfirst;
+	alpm_list_t *ignorepkg;
+	alpm_list_t *ignoregrp;
+	alpm_list_t *noupgrade;
+	alpm_list_t *noextract;
 	char *xfercommand;
+
+	/* our connection to libalpm */
+	alpm_handle_t *handle;
+
+	alpm_list_t *explicit_adds;
+	alpm_list_t *explicit_removes;
 } config_t;
 
 /* Operations */
@@ -106,7 +125,10 @@ enum {
 	OP_NEEDED,
 	OP_ASEXPLICIT,
 	OP_ARCH,
-	OP_PRINTFORMAT
+	OP_PRINTFORMAT,
+	OP_GPGDIR,
+	OP_RECURSIVE,
+	OP_DBONLY
 };
 
 /* clean method */
@@ -121,6 +143,8 @@ extern config_t *config;
 config_t *config_new(void);
 int config_free(config_t *oldconfig);
 
+int config_set_arch(const char *arch);
+int parseconfig(const char *file);
 #endif /* _PM_CONF_H */
 
 /* vim: set ts=2 sw=2 noet: */
