@@ -20,23 +20,20 @@
 
 #include "config.h"
 
-#include <stdlib.h>
 #include <stdio.h>
-#include <string.h>
 
 #include <alpm.h>
 #include <alpm_list.h>
 
 /* pacman */
 #include "pacman.h"
-#include "util.h"
 #include "conf.h"
 
 int pacman_deptest(alpm_list_t *targets)
 {
 	alpm_list_t *i;
 	alpm_list_t *deps = NULL;
-	pmdb_t *localdb = alpm_option_get_localdb();
+	alpm_db_t *localdb = alpm_option_get_localdb(config->handle);
 
 	for(i = targets; i; i = alpm_list_next(i)) {
 		char *target = alpm_list_getdata(i);
@@ -47,7 +44,7 @@ int pacman_deptest(alpm_list_t *targets)
 	}
 
 	if(deps == NULL) {
-		return(0);
+		return 0;
 	}
 
 	for(i = deps; i; i = alpm_list_next(i)) {
@@ -56,7 +53,7 @@ int pacman_deptest(alpm_list_t *targets)
 		printf("%s\n", dep);
 	}
 	alpm_list_free(deps);
-	return(127);
+	return 127;
 }
 
 /* vim: set ts=2 sw=2 noet: */
