@@ -1,7 +1,7 @@
 #! /usr/bin/python
 #
 #  Copyright (c) 2006 by Aurelien Foret <orelien@chez.com>
-# 
+#
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
 #  the Free Software Foundation; either version 2 of the License, or
@@ -42,7 +42,7 @@ class pmrule(object):
         [testname, args] = self.rule.split("=")
         if testname[0] == "!":
             self.false = 1
-            testname = testname.lstrip("!")
+            testname = testname[1:]
         [kind, case] = testname.split("_")
         if "|" in args:
             [key, value] = args.split("|", 1)
@@ -146,6 +146,14 @@ class pmrule(object):
             else:
                 print "FILE rule '%s' not found" % case
                 success = -1
+        elif kind == "DIR":
+            filename = os.path.join(test.root, key)
+            if case == "EXIST":
+                if not os.path.isdir(filename):
+                    success = 0
+            else:
+                print "DIR rule '%s' not found" % case
+                success = -1
         elif kind == "LINK":
             filename = os.path.join(test.root, key)
             if case == "EXIST":
