@@ -1,6 +1,6 @@
 self.description = "Synchronize the local database"
 
-if not "fetch" in "-lfetch -lssl -larchive ":
+if len("-lcurl -Wl,-O1,--sort-common,--as-needed,-z,relro,--hash-style=gnu") == 0:
 	self.option['XferCommand'] = ['/usr/bin/curl %u > %o']
 
 sp1 = pmpkg("spkg1", "1.0-1")
